C************************************************************************
C   PROGRAM SnCal CALCULATE THE NORMALIZED SLIDING DISTANCE FOR A MISSION
C       AND CALCUATE DELTA ENERGY FOR A BRAKE SCHEDULE
C
C      DEVELOPED BY ANGELES CREST ENGINEERING INC
C
C   DATE:       MAY 2013
C
C   SUBPROGRAMS:BRAKEPADAREACAL, BRAKEPERFORMANCE, BRAKEWEAR, 
C               DELTABRAKEENERGY, DLINERTIA, ENGTORQUECURVE1, 
C               ENGTORQUECURVE2, ENGTORQUECURVE3, INTERPOLATE, INPUT, 
C               SHIFTLOGIC, TRMNAT, WEAR
C
C   FILES:      1:SnCalDAT-OLD; 2:TerrainDAT-OLD;  3:SnCalOUT-NEW
C
C************************************************************************
      PROGRAM SnCal

      IMPLICIT NONE
      
      LOGICAL   FILETEST1

      INTEGER   MAXG, MAXPOWER, MAXSHIFT, MAXTERRAIN, MAXTERRAINCYCLE
      
      REAL      PI, G_S, G_C

      PARAMETER (MAXG = 6, MAXPOWER = 8, MAXSHIFT = 30000, 
     &          MAXTERRAIN = 20000, MAXTERRAINCYCLE = 10000)
     
      PARAMETER (PI = 3.1415926536, G_S = 32.174, G_C = 32.174 )
      
      INTEGER   BRAKECOUNT(MAXTERRAIN), BRAKEFAIL, CLUTCHFAIL(0:MAXG), 
     &          CLUTCHNUM, GEAR, GEAR_I, GSHIFT(0:MAXG), I, IOUT, J, K,   
     &          M, MAXGEAR(MAXTERRAIN), MAXGEAR_B(MAXTERRAIN),  MODEL, 
     &          NGEAR, NTERRAIN, TERRAINCYCLE

      DOUBLE PRECISION  ENGTORQUECURVE1, ENGTORQUECURVE2, 
     &                  ENGTORQUECURVE3
      
      REAL      ACCEL(MAXTERRAIN), ACCEL_FAULT(MAXTERRAINCYCLE), 
     &          ACCELMAX, BRAKE_D(MAXTERRAINCYCLE), BRAKE_V(MAXTERRAIN), 
     &          BRAKEDIST_FAULT(MAXTERRAINCYCLE), 
     &          BRAKEENERGY(MAXTERRAINCYCLE), DELTABRAKEENERGY, 
     &          BRAKETORQUE(MAXTERRAINCYCLE), 
     &          BRAKEWEAR, BRAKEWEARFRACTION(MAXTERRAINCYCLE), 
     &          CGFORCE(MAXTERRAIN), D(MAXTERRAINCYCLE, 0:MAXG), 
     &          DELTAX(MAXTERRAIN),  
     &          DELTAY(MAXTERRAIN), DOWNSHIFTENGSPD, DRAG(MAXTERRAIN), 
     &          DSTORQUE(MAXTERRAIN), ENGSPDIDLE,
     &          ENGSPD(MAXTERRAIN), ENGSPDMAX, ENGSPDMAXTOR,   
     &          ENGSPDMIN, ENGTORQUEREQ_G(0:MAXG, MAXTERRAIN) , FDRATIO,    
     &          FDEFFICIENCY, CFORCE, F_TORQUE(MAXTERRAINCYCLE, 0:MAXG), 
     &          F_TORQUET(MAXTERRAINCYCLE, 0:MAXG), G, 
     &          GEFFICIENCY(0:MAXG), GRADE(MAXTERRAIN), GRATIO(0:MAXG), 
     &          MAXCONVERTOR, MAXENGSPD_G(0:MAXG, MAXTERRAIN), 
     &          MAXENGSPD(MAXTERRAIN), MAXFORCE_PAD,
     &          MAXSPROCKETSPD_G(0:MAXG, MAXTERRAIN), 
     &          MAXSPD(MAXTERRAIN), MAXTORQUE, MAXTORQUEIDLESPD, 
     &          MAXTORQUEMAXSPD, 
     &          MAXTORQUEMINSPD, MINENGSPD_G(0:MAXG, MAXTERRAIN),  
     &          MAXVSPD(MAXTERRAIN), MAXVSPD_G(0:MAXG, MAXTERRAIN), 
     &          MINENGSPD(MAXTERRAIN), MINVSPD(MAXTERRAIN), 
     &          MINSPROCKETSPD_G(0:MAXG, MAXTERRAIN),   
     &          MINVSPD_G(0:MAXG, MAXTERRAIN), MUI_BRAKE, MUI_I, 
     &          ROTORADIUS, ROTORSPDBRAKE(MAXTERRAIN),
     &          ROTORSPD(MAXTERRAIN), SHIFTTIME, SLOPE(MAXTERRAIN),  
     &          SPROCKETRADIUS, SPROCKETSPD(MAXTERRAIN),  
     &          SPROCKETTORQUE(MAXTERRAIN), TOPSPD, TORQUE(MAXTERRAIN),  
     &          TORQUEREQ(MAXTERRAIN), TRACKDRAG,
     &          TRACKFORCE(MAXTERRAIN), SN(0:MAXG), TOTALBRAKEENERGY, 
     &          TRACKFRICTION, WEAR, VSPD(MAXTERRAIN),
     &          WEARFRACTION(MAXTERRAINCYCLE, 0:MAXG), TANK_MASS

      COMMON/IOUT/    MODEL
           
C ** OPEN INPUT AND OUTPUT FILES ==============================================
      OPEN (1,  FILE = 'SnCalDAT', STATUS = 'OLD')
      OPEN (2,  FILE = 'TerrainDAT', STATUS = 'OLD')
      
      INQUIRE (FILE = 'SnCalOUT', EXIST = FILETEST1)
      IF (FILETEST1 .EQV. .TRUE.) THEN
        OPEN (3,  FILE = 'SnCalOUT', STATUS = 'OLD')
        CLOSE (UNIT=3, STATUS='DELETE')
      ENDIF  
       
      OPEN (3,  FILE = 'SnCalOUT', STATUS = 'NEW')
       
C===============================================================================
      CALL  INPUT   ( ACCEL, ACCELMAX, BRAKECOUNT, BRAKE_V, CLUTCHNUM,    
     &          NGEAR, NTERRAIN, DRAG, ENGSPDIDLE, ENGSPDMAX, ENGSPDMIN,  
     &          CFORCE, FDRATIO, FDEFFICIENCY, G, GEAR_I, GEFFICIENCY, 
     &          GRATIO, MAXCONVERTOR, MAXFORCE_PAD, MAXTORQUE,  
     &          MAXTORQUEIDLESPD, MAXTORQUEMAXSPD, MAXTORQUEMINSPD,  
     &          MUI_BRAKE, MUI_I, ROTORADIUS, SLOPE, SPROCKETRADIUS, 
     &          SHIFTTIME, TANK_MASS, TERRAINCYCLE, TOPSPD, TRACKDRAG, 
     &          TRACKFRICTION, VSPD )

C   INITIALIZE VARIABLE     
      DO 20 I = 1, MAXTERRAIN
        TRACKFORCE(I) = 0.0
        SPROCKETTORQUE(I) = 0.0
        MAXSPD(I) = 0.0
        ENGSPD(I) = 0.0
        GRADE(I) = 0.0
   20 CONTINUE

      DO 30 I = 0, MAXG
        DO 40 J = 1, MAXTERRAIN
            ENGTORQUEREQ_G(I, J) = 0.0
            MAXENGSPD_G(I, J) = 0.0
            MINENGSPD_G(I, J) = 0.0
            MAXSPROCKETSPD_G(I, J) = 0.0
            MINSPROCKETSPD_G(I, J) = 0.0
   40   CONTINUE
   30 CONTINUE

      DO 50 I = 0, MAXG
        GSHIFT(I) = 0
        SN(I)= 0.0
        CLUTCHFAIL(I) = MAXTERRAIN
   50 CONTINUE

C   SET INITIAL GEAR
      GEAR = GEAR_I

C   CALCULATE ENG SPEED AT MAX TORQUE
      ENGSPDMAXTOR = ENGTORQUECURVE1(MAXTORQUE)

      WRITE(3,*) ''
      WRITE(3,*) ''
      WRITE(3,*) '                                 TERRAIN BLOCK CHECK '
      WRITE(3,*) '*****************************************************'
      
C===============================================================================
C   CALCULATE TERRAIN BLOCK DATA, OBTAIN MAXGEAR FOR EACH COURSE, 
C       CHECK FOR ILLEGAL VEHICLE SPEED
C
C===============================================================================
      DO 500, K = 1, NTERRAIN

C   INITIALIZE VARIABLE
        MAXGEAR(K) = 0

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   CALCULATE TORQUE LIMIT FOR COURSE (K)    
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

C   CALCULATE SLOPE (RAD), GRADE (%)
        SLOPE(K) = SLOPE(K) * PI / 180.0
        GRADE(K) = TAN ( SLOPE (K) ) * 100.0
        
C   CALCULATE CG FORCE
        CGFORCE(K) = TANK_MASS * SIN( SLOPE(K) )

C   CALCULATE TRACK FORCE
      TRACKFORCE(K) = DRAG(K) * TANK_MASS + TRACKFRICTION + CGFORCE(K)
      
C   CALCULATE REQUIRED SPROCKET OUTPUT TORQUE
        SPROCKETTORQUE(K) = TRACKFORCE(K) * SPROCKETRADIUS
                
C   CALCULATE DRIVESHAFT TORQUE
        DSTORQUE(K) = SPROCKETTORQUE(K)/ ( FDRATIO * FDEFFICIENCY )       
        
        IF ( DSTORQUE(K) .LT. 0.0 ) THEN
                DSTORQUE(K) = 0.0
        ENDIF
 
C   CALCULATE INPUT TORQUE REQUIRED FOR INDIVIDUAL GEAR
        DO 100 I = 0, NGEAR
            ENGTORQUEREQ_G(I, K) = SPROCKETTORQUE(K) / 
     &                  ( GRATIO(I) *  GEFFICIENCY(I) ) /
     &                  ( FDRATIO * FDEFFICIENCY ) 
            IF ( ENGTORQUEREQ_G(I, K) .LT. 0.0 ) THEN
                ENGTORQUEREQ_G(I, K) = 0.0
            ENDIF
  100   CONTINUE

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   CALCULATE VEHICLE SPEED LIMIT FOR COURSE (K)    
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

C   CALCULATE MAX/MIN VEHICLE SPEED FOR INDIVIDUAL GEAR FOR THE CURRENT TERRAIN
        DO 110 I = 0, NGEAR
        
C   CALCULATE MAX/MIN ENGINE SPEED FOR INDIVIDUAL GEAR FOR THE CURRENT TERRAIN        
            IF (ENGTORQUEREQ_G(I, K) .LE. MAXTORQUE) THEN
                IF (ENGTORQUEREQ_G(I, K) .GT. MAXTORQUEMAXSPD) THEN
                    MAXENGSPD_G(I, K) = 
     &                  ENGTORQUECURVE1( ENGTORQUEREQ_G(I, K) )
                ELSE
                    MAXENGSPD_G(I, K) = ENGSPDMAX
                ENDIF
        
                IF (ENGTORQUEREQ_G(I, K) .GT. MAXTORQUEMINSPD) THEN
                    MINENGSPD_G(I, K) = 
     &                  ENGTORQUECURVE2( ENGTORQUEREQ_G(I, K) )
                ELSE IF ((ENGTORQUEREQ_G(I, K) .GT. MAXTORQUEIDLESPD) 
     &              .AND. (ENGTORQUEREQ_G(I, K) .LT. MAXTORQUEMINSPD) )
     &          THEN
                    MINENGSPD_G(I, K) = 
     &                  ENGTORQUECURVE3( ENGTORQUEREQ_G(I, K) )
                ELSE
                    MINENGSPD_G(I, K) = ENGSPDIDLE
                ENDIF
            ELSE 
                MAXENGSPD_G(I, K) = 0.0
                MINENGSPD_G(I, K) = 1.0e20
            ENDIF
         
C   CALCULATE MAX/MIN SPROCKET SPEED (RPM) FOR INDIVIDUAL GEAR FOR THE CURRENT TERRAIN
            MAXSPROCKETSPD_G(I, K) = 
     &          MAXENGSPD_G(I, K) / GRATIO(I) / FDRATIO
            MINSPROCKETSPD_G(I, K) = 
     &          MINENGSPD_G(I, K) / GRATIO(I) / FDRATIO

C   CALCULATE MAX/MIN VEHICLE SPEED (MPH) FOR INDIVIDUAL GEAR FOR THE CURRENT TERRAIN
            MAXVSPD_G(I, K) = 
     &          MAXSPROCKETSPD_G(I, K) *60*2*PI* SPROCKETRADIUS / 5280.0
            MINVSPD_G(I, K) = 
     &          MINSPROCKETSPD_G(I, K) *60*2*PI* SPROCKETRADIUS / 5280.0

  110   CONTINUE
  
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   MAX GEAR CALCULATION FOR COURSE (K)    
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

C   FIND OUT THE HIGHEST GEAR FOR THE CURRENT TERRAIN
        DO 200 I = 1, NGEAR
            IF ( VSPD(K) .GE. MINVSPD_G(I , K) ) THEN
                MAXGEAR(K) = MAXGEAR(K) + 1
            ENDIF
  200   CONTINUE
  
C   FIND OUT THE HIGHEST GEAR FOR THE CURRENT TERRAIN AFTER BRAKING 
        DO 205 I = 1, NGEAR
            IF ( BRAKE_V(K) .GE. MINVSPD_G(I , K) ) THEN
                MAXGEAR_B(K) = MAXGEAR_B(K) + 1
            ENDIF
  205   CONTINUE
  
C   CALCULATE THE ENGINE TORQUE, MAX/MIN ENGINE/VEHICLE SPEED FOR THE CURRENT TERRAIN
        TORQUEREQ(K) = ENGTORQUEREQ_G(MAXGEAR(K), K)
        MAXVSPD(K) = MAXVSPD_G(MAXGEAR(K), K) 
        MINVSPD(K) = MINVSPD_G(MAXGEAR(K), K) 
        MAXENGSPD(K) = MAXENGSPD_G(MAXGEAR(K), K)
        MINENGSPD(K) = MINENGSPD_G(MAXGEAR(K), K)

C   DETERMINE IF THE REQUIRED VEHICLE SPEED IS POSSIBLE (ENGINE TORQUE LIMIT) FOR THE CURRENT TERRAIN
        IF ( VSPD(K) .GT. MAXVSPD(K) ) THEN
            VSPD(K) = MAXVSPD(K)
        WRITE (3, *) "     Input Vehicle Speed too high for Course #", K 
        WRITE (3, *) "                       Lower Vehicle Speed to ", 
     &      VSPD(K)
        WRITE (3, *) ""       
        ENDIF

C   DETERMINE IF THE FINAL BRAKING VEHICLE SPEED IS POSSIBLE
        IF ( BRAKE_V(K) .GT. VSPD(K) ) THEN
            BRAKECOUNT(K) = 0
        WRITE (3, *)"Final Brake Vehicle Speed too high for Course #", K 
        WRITE (3, *) "       Set Brake Count For This Course to Zero"
        ENDIF

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   SPEED CALCULATION (RPM) FOR COURSE (K)    
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        SPROCKETSPD(K) = VSPD(K) /60.0/2.0/PI/ SPROCKETRADIUS * 5280.0
        
        ROTORSPD(K) = SPROCKETSPD(K) * FDRATIO
        
        ROTORSPDBRAKE(K) = BRAKE_V(K) / 60.0 / 2.0 / PI / SPROCKETRADIUS 
     &              * 5280.0 * FDRATIO
        
        ENGSPD(K) = ROTORSPD(K) * GRATIO( MAXGEAR(K) ) 
        
        IF ( ENGSPD(K) .LT. ENGSPDIDLE) THEN
            ENGSPD(K) = ENGSPDIDLE
        ENDIF

  500 CONTINUE

C===============================================================================
C   PERFORM SHIFT LOGIC/BRAKE SCHEDULE 
C
C===============================================================================
      TOTALBRAKEENERGY = 0.0
      
C   TERRAIN BLOCK LOOP
      DO 700, J = 1, TERRAINCYCLE
      BRAKEENERGY(J) = 0.0

C   COURSE LOOP      
        DO 600, K = 1, NTERRAIN

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   PERFORM SHIFT LOGIC (SHIFT TO CORRECT GEAR FOR CURRENT COURSE
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            CALL SHIFTLOGIC (ENGSPDMAX, GEAR, GRATIO, GSHIFT,   
     &          MAXGEAR(K), SN, SHIFTTIME)   

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   BRAKE CALCULATION
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            IF ( BRAKECOUNT(K) .GT. 0 ) THEN
C   ENERGY DISSIPATED DUE TO BRAKING FOR CURRENT COURSE   
                TOTALBRAKEENERGY = TOTALBRAKEENERGY +
     &              DELTABRAKEENERGY(ACCEL(K), ROTORSPD(K), 
     &              ROTORSPDBRAKE(K), TANK_MASS, SLOPE(K), VSPD(K), 
     &              BRAKE_V(K) ) * BRAKECOUNT(K)

C   PERFORM GEAR SHIFT DUE TO BRAKING
                DO 210 I = 1, BRAKECOUNT(K)
C   SPEED DOWN
                    CALL SHIFTLOGIC (ENGSPDMAX, GEAR, GRATIO, GSHIFT,   
     &                  MAXGEAR_B(K), SN, SHIFTTIME)   
C   SPEED UP
                    CALL SHIFTLOGIC (ENGSPDMAX, GEAR, GRATIO, GSHIFT,   
     &                  MAXGEAR(K), SN, SHIFTTIME)           
  210           CONTINUE
            ENDIF
     
  600   CONTINUE

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   BRAKE WEAR CALCULATIONS (ACCUMULATED UP TO CURRENT TERRAIN BLOCK)
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   CUMULATIVE ENERGY DISSIPATED DUE TO BRAKING    
        BRAKEENERGY(J) = TOTALBRAKEENERGY

C   CALCULATE BRAKE DECELERATION CAPABILITY AND STOPPING DISTANCE 
C       FROM MAX VELOCITY AFTER CURRENT TERRAIN BLOCK
        CALL BRAKEPERFORMANCE(ACCEL_FAULT(J), BRAKE_D(J), 
     &      BRAKEDIST_FAULT(J), BRAKEENERGY(J), BRAKETORQUE(J),   
     &      BRAKEWEARFRACTION(J), FDRATIO, MAXFORCE_PAD, MUI_BRAKE, 
     &      ROTORADIUS, SPROCKETRADIUS, TANK_MASS, TOPSPD, TRACKDRAG)

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   CLUTCH WEAR CALCULATIONS (ACCUMULATED UP TO CURRENT TERRAIN BLOCK)
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   NUMBER OF SHIFT, SN, WEARFRACTION, DAMAGE, TORQUE FOR EACH GEAR
        DO 250 I = 0, NGEAR
            WEARFRACTION(J, I) = WEAR(SN(I))
            IF (WEARFRACTION(J, I) .LE. 0.6) THEN
                D(J, I) = WEARFRACTION(J, I) / 3.0
            ELSEIF (WEARFRACTION(J, I) .LE. 1.0) THEN
                D(J, I) = 0.2 + 2.0 * (WEARFRACTION(J, I) - 0.6 )
            ELSE
                D(J, I) = 1.0
            ENDIF    
            F_TORQUE(J, I) = CFORCE * ( 1.0 - D(J, I) ) * MUI_I * G
            F_TORQUET(J, I) = F_TORQUE(J, I) * FLOAT(CLUTCHNUM)
  250   CONTINUE

C   CHECK FOR SLIPPING IN CLUTCH
        DO 255 K = 1, NTERRAIN
            IF ( F_TORQUET(J, MAXGEAR(K)) .GE. TORQUEREQ(K) ) THEN
                CLUTCHFAIL(MAXGEAR(K)) = J
            ENDIF  
  255   CONTINUE
  
  700 CONTINUE    

C===============================================================================
C   OUTPUT
C
C===============================================================================

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   BRAKE RESULT
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      IF ( (MODEL .EQ. 0) .OR. (MODEL .EQ. 2) ) THEN
C   WRITE FINAL DELTAENERGY, WEAR FRACTION, DAMAGE, TOTAL BRAKE TORQUE, ETC TO OUTPUT FILE
       WRITE(3,*)''
       WRITE(3,*)''
       WRITE(3,*)'                                PROGRAM OUTPUT(BRAKE)'
       WRITE(3,*)'*****************************************************'
       WRITE(3, 804)  

        DO 310 J = 1, TERRAINCYCLE
            WRITE (3, 805) J, BRAKEENERGY(J), BRAKEWEARFRACTION(J), 
     &          BRAKE_D(J), BRAKETORQUE(J), ACCEL_FAULT(J), 
     &          BRAKEDIST_FAULT(J)
            IF (ACCEL_FAULT(J) .LE. ACCELMAX ) THEN
               BRAKEFAIL = J 
            ENDIF
  310   CONTINUE

        IF (BRAKEFAIL .LT. TERRAINCYCLE) THEN
           WRITE(3,*)''
           WRITE(3,*)'BRAKE PAD FAILED TO MEET THE REQUIREMENT'
           WRITE(3,*)'   AFTER FINISHED TERRAIN BLOCK #', BRAKEFAIL
        ENDIF
           
      ENDIF
      
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C   CLUTCH RESULT
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      IF ( (MODEL .EQ. 0) .OR. (MODEL .EQ. 1) ) THEN
C   WRITE FINAL SN, WEAR FRACTION, DAMAGE, FRICTIONAL TORQUE FOR EACH GEAR TO OUTPUT FILE
       WRITE(3,*)''
       WRITE(3,*)''
       WRITE(3,*)'                               PROGRAM OUTPUT(CLUTCH)'
       WRITE(3,*)'*****************************************************'
       WRITE(3, 800)  
          
        DO 300 I = 0, NGEAR
            WRITE (3, 801) I, GSHIFT(I), SN(I), 
     &          WEARFRACTION(TERRAINCYCLE, I), D(TERRAINCYCLE, I), 
     &          F_TORQUE(TERRAINCYCLE, I), F_TORQUET(TERRAINCYCLE, I)
  300   CONTINUE

        WRITE (3, *) '** GEAR 0 = TORQUE CONVERTER CLUTCH'
        
C   WRITE ENGINE TORQUE/SPEED, MAXIMUM VEHICLE SPEED TO OUTPUT FILE
       WRITE(3,*)''
       WRITE(3,*)''
       WRITE(3,*)'                       PROGRAM OUTPUT (TERRAIN BLOCK)'
       WRITE(3,*)'*****************************************************'

        WRITE (3, 802)  

        DO 400 K = 1, NTERRAIN
            WRITE (3, 803) K, MAXGEAR(K), TORQUEREQ(K), DSTORQUE(K), 
     &                  DSTORQUE(K)/2, ENGSPD(K), VSPD(K)
  400   CONTINUE

            WRITE(3,*)''

        DO 405 I = 0, NGEAR
            IF (CLUTCHFAIL(I) .LT. TERRAINCYCLE) THEN
            WRITE(3,*)'CLUTCH FOR GEAR', I
            WRITE(3,*)'   STARTED TO SLIP ON CERTAIN COURSE SEGMENT'
            WRITE(3,*)'   AFTER FINISHED TERRAIN BLOCK #', CLUTCHFAIL(I)
        ENDIF
           
  405   CONTINUE
      ENDIF
      
  800 FORMAT(1X,'GEAR   TOTAL #     SN        WEARFRACTION      DAMAGE
     &         FRICTIONAL TORQUE', / , 7X ,'OF SHIFTS'
     &   ,45X, '(PER SURFACE)     (TOTAL)')
  
  801 FORMAT(3X, I1, I10, 2X, F10.1,5X, F10.8, 5X, F10.8, 5X, F10.4, 5X, 
     &       F10.4)

  802 FORMAT(1X,
     &'COURSE MAX.  MIN.ENGINE DRIVESHAFT BULLGEAR ENGINE  VEHICLE',  
     &/ , 2X , 
     &  '#    GEAR   TORQUE     TORQUE     TORQUE   SPEED   SPEED ',/,
     & 9X , '     REQUIRED                          (rpm)   (mph)',/,
     & 15X, '(lb-ft)   (lb-ft)     (lb-ft)')
         
  803 FORMAT(3X, I3, 4X, I1, 2X, F8.1,  2X, F8.1, 2X, F8.1, 5X, F6.1, 
     & 5X, F5.1)

  804 FORMAT(1X,'TERRAIN   DELTA       WEARFRACTION   DAMAGE     MAX', 
     &  '        MAX G     STOPING', / , 1X ,'BLOCK #   ENERGY' 
     &   , 32X, 'BRAKE                 DISTANCE',/, 
     &   49X, 'TORQUE             FROM MAX SPD')
 
  805 FORMAT(3X, I3, E13.5, 2X, E13.5, 5X, F7.5, 2X, F9.2, 2X, F8.4, 2X, 
     &  F9.4)
  
      STOP

      END

C************************************************************************
C   SUBROUTINE TRMNAT HANDLES THE TERMINATION OF THE PROGRAM RUN WHEN
C       ONE OF THE PROGRAM'S ASSUMPTIONS HAVE BEEN VIOLATED
C
C************************************************************************
      SUBROUTINE TRMNAT

      WRITE(3,*) 'PROGRAM EXECUTION TERMINATED'
      STOP
      END  

C************************************************************************
C   SUBROUTINE INPUT READS IN THE DATA AND ECHOES IT 
C
C   DATE :      JANUARY 2013
C
C************************************************************************
      SUBROUTINE INPUT ( ACCEL, ACCELMAX, BRAKECOUNT, BRAKE_V, 
     &          CLUTCHNUM, NGEAR, NTERRAIN, DRAG, ENGSPDIDLE, ENGSPDMAX,   
     &          ENGSPDMIN, CFORCE, FDRATIO, FDEFFICIENCY, G, GEAR_I,   
     &          GEFFICIENCY, GRATIO, MAXCONVERTOR, MAXFORCE_PAD,  
     &          MAXTORQUE, MAXTORQUEIDLESPD,MAXTORQUEMAXSPD,  
     &          MAXTORQUEMINSPD, MUI_BRAKE, MUI_I,ROTORADIUS, SLOPE, 
     &          SPROCKETRADIUS, SHIFTTIME, TANK_MASS, TERRAINCYCLE, 
     &          TOPSPD, TRACKDRAG, TRACKFRICTION, VSPD )
 
      INTEGER   MAXBLF, MAXG, MAXPOWER, MAXTERRAIN

      REAL      PI, G_S, G_C
      
      PARAMETER (MAXBLF = 35, MAXG = 6, MAXPOWER = 8, 
     &          MAXTERRAIN = 20000)
      
      PARAMETER (PI = 3.1415926536, G_S = 32.174, G_C = 32.174 )

      INTEGER   BLFPOS, BRAKECOUNT(MAXTERRAIN), CLUTCHNUM, FILNUM(4), 
     &          BRAKEFILNUM(5), GEAR_I, I, J, MODEL,     
     &          NBLIFE, NGEAR, NTERRAIN, NTIME, POWER1, POWER2, POWER3, 
     &          POWER4, TERRAINCYCLE

      DOUBLE PRECISION  COEF1(0:MAXPOWER), COEF2(0:MAXPOWER),
     &                  COEF3(0:MAXPOWER), COEF4(0:MAXPOWER)

      REAL      A, AREA, ACCEL(MAXTERRAIN), ACCELMAX, BLFPER(MAXBLF), 
     &          BRAKEPADAREA, BRAKEPADAREA_A(5), BRAKEPADMINAREA, 
     &          BRAKEPADPRESS, BRAKE_V(MAXTERRAIN), CFORCE, 
     &          BRAKEWEARTABLE(5, MAXBLF, 5:14), 
     &          CLUTCHWEARTABLE(4, MAXBLF, 0:8),   
     &          DL_MINSPD, DRAG(MAXTERRAIN), ENGFACTOR, ENGSPDIDLE, 
     &          ENGSPDMAX, ENGSPDMIN, FDRATIO, FDEFFICIENCY,    
     &          FORCE, G, GEFFICIENCY(0:MAXG), GRATIO(0:MAXG), I_ROTOR, 
     &          ID, M_ROTOR, MAXBRAKEDIST, MAXCONVERTOR, MAXFORCE_PAD, 
     &          MAXTORQUE, MAXTORQUEIDLESPD, MAXTORQUEMAXSPD, 
     &          MAXTORQUEMINSPD, MUI_BRAKE, MUI_I, NFORCE, OD,   
     &          ROTORDEN, ROTORID, ROTORADIUS, ROTOROD, ROTORTHICK, RM,  
     &          SHIFTTIME, SLOPE(MAXTERRAIN), SPROCKETRADIUS, T_A(4),  
     &          T_FORCE(4), TANK_MASS, TOPSPD, TORQUERATING, 
     &          TRACKDRAG, TRACKFRICTION, TS, VSPD(MAXTERRAIN)
            
      CHARACTER*20 BRAKETABLENAME(5), TABLENAME(4)
      
      CHARACTER*20 ROTORMAT

      LOGICAL   FTEST
   
      DATA (TABLENAME(I), I = 1, 4) /
     &     'ParaTable1', 'ParaTable2', 'ParaTable3', 'ParaTable4'/
     
      DATA (BRAKETABLENAME(I), I = 1, 5) /
     &     'BRAKEWEARTable1', 'BRAKEWEARTable2', 'BRAKEWEARTable3',
     &     'BRAKEWEARTable4', 'BRAKEWEARTable5'/
     
      DATA (FILNUM(I), I = 1, 4) /
     &     11, 12, 13, 14 /

      DATA (BRAKEFILNUM(I), I = 1, 5) /
     &     21, 22, 23, 24, 25 /
             
      COMMON/CURVE1/   COEF1, POWER1  

      COMMON/CURVE2/   COEF2, POWER2
   
      COMMON/CURVE3/   COEF3, POWER3

      COMMON/CURVE4/   COEF4, POWER4, DL_MINSPD
      
      COMMON/FACTOR/   ENGFACTOR
      
      COMMON/BRAKEENERGY/ I_ROTOR

      COMMON/CLUTCHMAP/  A, FORCE, T_A, T_FORCE, CLUTCHWEARTABLE
      
      COMMON/BRAKEMAP/   BRAKEPADAREA, BRAKEPADAREA_A, BRAKEWEARTABLE

      COMMON/MAP/      BLFPOS
      
      COMMON/IOUT/    MODEL
      
      READ(1,*) MODEL
      WRITE(3,*) '                TYPE OF COURSE/MODEL =', MODEL
      WRITE(3,*) '*****************************************************'
      WRITE(3,*) '                0 = BRAKE + CLUTCH'
      WRITE(3,*) '                1 = CLUTCH ONLY'
      WRITE(3,*) '                2 = BRAKE ONLY'

      READ(1,*) NTERRAIN
      WRITE(3,*) ''
      WRITE(3,*) '                             TERRAIN INPUT PARAMETERS'
      WRITE(3,*) '*****************************************************'
      
      WRITE(3,*) '                   Number of Courses =', NTERRAIN

      IF (NTERRAIN .GT. MAXTERRAIN) THEN
         WRITE(3,*) 'ERROR:  TOO MANY COURSES'
         CALL TRMNAT
      ENDIF

      READ(1,*) TERRAINCYCLE  
      WRITE(3,*) '            Number of Courses Cycles =', TERRAINCYCLE
      
      WRITE(3,*) ''
      WRITE(3,*) '                  TANK SPECIFICATION INPUT PARAMETERS'
      WRITE(3,*) '*****************************************************'
      
      READ(1,*) TRACKFRICTION
      WRITE(3,*)'                       Track Friction =', TRACKFRICTION

      READ(1,*) TANK_MASS
      WRITE(3,*) '                          Tank Weight =', TANK_MASS
      
      READ(1,*) TRACKDRAG
      WRITE(3,*) '                   Typical Track Drag =', TRACKDRAG     
      
      READ(1,*) SPROCKETRADIUS
      WRITE(3,*)'                      Sprocket Radius =',SPROCKETRADIUS
      
      READ(1,*) ACCELMAX
      WRITE(3,*)'Min. Brake Deceleration Capability (g)=',ACCELMAX
      
      READ(1,*) MAXBRAKEDIST
      WRITE(3,*)'  Max. Braking Distance @ Max. Decel. =',MAXBRAKEDIST
      
      READ(1,*) TOPSPD
      WRITE(3,*)'                  Max. Vehicle Speed. =', TOPSPD

      WRITE(3,*) ''
      WRITE(3,*)'            DRIVELINE ROTATIONAL INERTIA (%ADDED MASS)'
      WRITE(3,*)'                                        VS SPEED CURVE'
      WRITE(3,*) '*****************************************************'
      
      READ(1,*) DL_MINSPD
      WRITE(3,*) '  Minimum Speed for Max % Added Mass =', DL_MINSPD 
      
      WRITE(3,*) '                         % Added Mass =              '
      
      READ(1,*) POWER4
      
      IF (POWER1 .GT. MAXPOWER) THEN
         WRITE(8,*) 'ERROR: CURVE WITH HIGHER THAN MAX ORDER'
         CALL TRMNAT
      ENDIF
      
      DO 47 I = 0, POWER4-1
        READ(1,*) COEF4(I)
        WRITE(3,903) COEF4(I), POWER4-I
   47 CONTINUE
           
      READ(1,*) COEF4(POWER4)
      WRITE(3,905) COEF4(POWER4)
      
      WRITE(3,*) ''
      WRITE(3,*) '              GEAR BOX SPECIFICATION INPUT PARAMETERS'
      WRITE(3,*) '*****************************************************'

      READ(1,*) NGEAR
      WRITE(3,*) '                 Total Number Of Gear =', NGEAR
      
      IF (NGEAR .GT. MAXTERRAIN) THEN
         WRITE(3,*) 'ERROR:  TOO MANY GEARS'
         CALL TRMNAT
      ENDIF
      
      READ(1,*) MAXCONVERTOR
      WRITE(3,*) '      Torque Convertor Multiplication =', MAXCONVERTOR
            
      DO 10 I = 1, NGEAR
        READ(1,*) GRATIO(I)
   10 CONTINUE
   
      DO 20 I = 1, NGEAR
        READ(1,*) GEFFICIENCY(I)
   20 CONTINUE

      GRATIO(0) = GRATIO(1) * MAXCONVERTOR
      GEFFICIENCY(0) = GEFFICIENCY(1)

      WRITE(3,900)
      I = 0
      WRITE(3,901)I, GRATIO(I), GEFFICIENCY(I)

      DO 30 I = 1, NGEAR
        WRITE(3,901)I, GRATIO(I), GEFFICIENCY(I)
   30 CONTINUE
   
      WRITE (3, *) '** GEAR 0 = GEAR 1 with TORQUE CONVERTER'

      READ(1,*) FDRATIO
      WRITE(3,*) '                    Final Drive Ratio =', FDRATIO
      
      READ(1,*) FDEFFICIENCY
      WRITE(3,*) '               Final Drive Efficiency =', FDEFFICIENCY
      
      WRITE(3,*) ''
      WRITE(3,*) '                 ENGINE TORQUE CURVE INPUT PARAMETERS'
      WRITE(3,*) '*****************************************************'

      READ(1,*) ENGSPDIDLE
      WRITE(3,*) '                     Engine Idle Speed =', ENGSPDIDLE
     
      READ(1,*) ENGSPDMIN
      WRITE(3,*) '                  Minimum Engine Speed =', ENGSPDMIN
      
      READ(1,*) ENGSPDMAX
      WRITE(3,*) '                  Maximum Engine Speed =', ENGSPDMAX
      
      READ(1,*) MAXTORQUE
      WRITE(3,*) '                 Engine Maximum Torque =', MAXTORQUE
      
      READ(1,*) MAXTORQUEMAXSPD
      WRITE(3,*) 'Maximum Torque at Maximum Engine Speed =', 
     &              MAXTORQUEMAXSPD

      READ(1,*) MAXTORQUEMINSPD
      WRITE(3,*) 'Maximum Torque at Minimum Engine Speed =', 
     &              MAXTORQUEMINSPD

      READ(1,*) MAXTORQUEIDLESPD
      WRITE(3,*) '   Maximum Torque at Idle Engine Speed =', 
     &              MAXTORQUEIDLESPD

      WRITE(3,*) ''
      WRITE(3,*) '  ENGINE TORQUE CURVE PARAMETERS (WITH SCALE FACTOR)'
      WRITE(3,*) '*****************************************************'

      READ(1,*) ENGFACTOR
      WRITE(3,*) '      Engine Torque Curve Scale Factor =', ENGFACTOR

      WRITE(3,*) '                     Engine Idle Speed =', ENGSPDIDLE
     
      WRITE(3,*) '                  Minimum Engine Speed =', ENGSPDMIN
      
      WRITE(3,*) '                  Maximum Engine Speed =', ENGSPDMAX
      
      MAXTORQUE = MAXTORQUE * ENGFACTOR
      WRITE(3,*) '                 Engine Maximum Torque =', MAXTORQUE
      
      MAXTORQUEMAXSPD = MAXTORQUEMAXSPD * ENGFACTOR
      WRITE(3,*) 'Maximum Torque at Maximum Engine Speed =', 
     &              MAXTORQUEMAXSPD

      MAXTORQUEMINSPD = MAXTORQUEMINSPD * ENGFACTOR
      WRITE(3,*) 'Maximum Torque at Minimum Engine Speed =', 
     &              MAXTORQUEMINSPD

      MAXTORQUEIDLESPD = MAXTORQUEIDLESPD * ENGFACTOR
      WRITE(3,*) '   Maximum Torque at Idle Engine Speed =', 
     &              MAXTORQUEIDLESPD

      WRITE(3,*) ''
      WRITE(3,*) '                       ENGINE TORQUE CURVE EQUATION 1'
      WRITE(3,*) '*****************************************************'
      WRITE(3,*) '                         Engine Speed =              '
      
      READ(1,*) POWER1
      
      IF (POWER1 .GT. MAXPOWER) THEN
         WRITE(8,*) 'ERROR: CURVE WITH HIGHER THAN MAX ORDER'
         CALL TRMNAT
      ENDIF
      
      DO 40 I = 0, POWER1-1
        READ(1,*) COEF1(I)
        WRITE(3,903) COEF1(I), POWER1-I
   40 CONTINUE
           
      READ(1,*) COEF1(POWER1)
      WRITE(3,904) COEF1(POWER1)

      WRITE(3,*) ''
      WRITE(3,*) '                       ENGINE TORQUE CURVE EQUATION 2'
      WRITE(3,*) '*****************************************************'
      WRITE(3,*) '                         Engine Speed =              '
      
      READ(1,*) POWER2
      
      IF (POWER2 .GT. MAXPOWER) THEN
         WRITE(8,*) 'ERROR: CURVE WITH HIGHER THAN MAX ORDER'
         CALL TRMNAT
      ENDIF
      
      DO 45 I = 0, POWER2-1
        READ(1,*) COEF2(I)
        WRITE(3,903) COEF2(I), POWER2-I
   45 CONTINUE
           
      READ(1,*) COEF2(POWER2)
      WRITE(3,904) COEF2(POWER2)

      WRITE(3,*) ''
      WRITE(3,*) '                       ENGINE TORQUE CURVE EQUATION 3'
      WRITE(3,*) '*****************************************************'
      WRITE(3,*) '                         Engine Speed =              '
      
      READ(1,*) POWER3
      
      IF (POWER3 .GT. MAXPOWER) THEN
         WRITE(8,*) 'ERROR: CURVE WITH HIGHER THAN MAX ORDER'
         CALL TRMNAT
      ENDIF
      
      DO 46 I = 0, POWER3-1
        READ(1,*) COEF3(I)
        WRITE(3,903) COEF3(I), POWER3-I
   46 CONTINUE
           
      READ(1,*) COEF3(POWER3)
      WRITE(3,904) COEF3(POWER3)

C   INPUT CLUTCH SPEC
      
      WRITE(3,*) ''
      WRITE(3,*) 'CLUTCH SPECIFICATION INPUT PARAMETERS (PER SURFACE)'
      WRITE(3,*) '*****************************************************'

      READ(1,*) TORQUERATING
      WRITE(3,*) '                Maximum Torque Rating =', TORQUERATING
      
      READ(1,*) NFORCE
      WRITE(3,*) '                 Maximum Normal Force =', NFORCE
      
      READ(1,*) TS
      WRITE(3,*) '   Shift Time at Maximum Normal Force =', TS
      
      READ(1,*) AREA
      WRITE(3,*) '           Clutch Friction Plate Area =', AREA  
      
      READ(1,*) OD
      WRITE(3,*) ' Clutch Friction Plate Outer Diameter =', OD
      
      READ(1,*) CLUTCHNUM
      WRITE(3,*) '   Number of Clutch Friction Surfaces =', CLUTCHNUM
      
      WRITE(3,*) ''
      WRITE(3,*) '                        CLUTCH DATA (FROM SIMULATION)'
      WRITE(3,*) '*****************************************************'

      READ(1,*) FORCE
      WRITE(3,*) '           Actual Clutch Normal Force =', FORCE
         
      CFORCE = FORCE
                  
      READ(1,*) GEAR_I
      WRITE(3,*) '                         Initial Gear =', GEAR_I
      
      WRITE(3,*) ''
      WRITE(3,*) '                            CLUTCH DATA (CALCULATION)'
      WRITE(3,*) '*****************************************************'
      
      SHIFTTIME = TS * (NFORCE / FORCE)
      WRITE(3,*) '             Actual Clutch Shift Time =', SHIFTTIME
      
      ID = SQRT(OD ** 2.0 - 4.0 * AREA / PI)
      RM = (OD + ID) / 4.0
      A = AREA / RM
      
      G = ( (OD / 12.0) ** 3.0 - (ID / 12.0) ** 3.0 ) 
     &    / 3.0 / ( (OD / 12.0) ** 2.0 - (ID / 12.0) ** 2.0 )

      MUI_I = TORQUERATING / NFORCE / G
      
      WRITE(3,*) ' Clutch Friction Plate Inner Diameter =', ID
      WRITE(3,*) '  Clutch Friction Plate Median Radius =', RM
      WRITE(3,*) '           Clutch Geometric Parameter =', A
      WRITE(3,*) '  Frictional Torque Geom. Parameter G =', G
      WRITE(3,*) '      Initial Coefficient of Friction =', MUI_I

      WRITE(3,*) ''
      WRITE(3,*) '                 BRAKE SPECIFICATION INPUT PARAMETERS'
      WRITE(3,*) '*****************************************************'
      
      READ(1,*) ROTORMAT
      WRITE(3,*) '           Brake Rotor Made Of ', ROTORMAT
      
      READ(1,*) ROTORDEN
      WRITE(3,*) '                  Brake Rotor Density =', ROTORDEN
      
      READ(1,*) ROTOROD
      WRITE(3,*) '           Brake Rotor Outer Diameter =', ROTOROD
      
      READ(1,*) ROTORID
      WRITE(3,*) '           Brake Rotor Inner Diameter =', ROTORID
      
      READ(1,*) ROTORTHICK
      WRITE(3,*) '                Brake Rotor Thickness =', ROTORTHICK
      
      READ(1,*) BRAKEPADAREA
      WRITE(3,*) '                 Total Brake Pad Area =', BRAKEPADAREA
      
      READ(1,*) BRAKEPADPRESS
      WRITE(3,*)'    Max. Allowable Brake Pad Pressure =', BRAKEPADPRESS    
      
      READ(1,*) MUI_BRAKE
      WRITE(3,*) ' Coeff. of Friction (Brake Pad/Rotor) =', MUI_BRAKE
      
      WRITE(3,*) ''
      WRITE(3,*) '                   BRAKE PAD/ROTOR DATA (CALCULATION)'
      WRITE(3,*) '*****************************************************'
      
      ROTORADIUS = ( ROTOROD + ROTORID ) / 4.0
      WRITE(3,*) '         Brake Rotor Effective Radius =', ROTORADIUS

      M_ROTOR = PI * ROTOROD * ROTOROD * ROTORTHICK * ROTORDEN / 4.0
      WRITE(3,*) '                     Brake Rotor Mass =', M_ROTOR

C  unit in Slug-ft^2      
      I_ROTOR = M_ROTOR * ROTOROD / 12.0 * ROTOROD / 12.0 / 8.0 / G_C
      WRITE(3,*) '       Brake Rotor Rotational Inertia =', I_ROTOR
      
C   Calculate the Minimum Brake Pad Area required 
      CALL   BRAKEPADAREACAL( ACCELMAX, BRAKEPADMINAREA, 
     &          BRAKEPADPRESS, FDRATIO, MUI_BRAKE, 
     &          ROTORADIUS, TANK_MASS, TRACKDRAG, SPROCKETRADIUS )

C   Check if input Brake pad area meet requirement 
      IF ( BRAKEPADAREA .LT. BRAKEPADMINAREA ) THEN
        BRAKEPADAREA = BRAKEPADMINAREA
        WRITE(3,*) ' '
        WRITE(3,*) ' The Input Brake Pad Area is too small to produce ' 
        WRITE(3,*) '        the minimum Brake Deceleration Capability ' 
        WRITE(3,*) '        Increase Brake Pad Area to ',BRAKEPADMINAREA
      ENDIF       
      
C   Calculate the Maximum Brake Force 
      MAXFORCE_PAD = BRAKEPADAREA * BRAKEPADPRESS             
      WRITE(3,*) ' '
      WRITE(3,*) '               Maximum Brake Force =', MAXFORCE_PAD

C   PARAMETER MAP DATA      
      READ(1,*) NBLIFE
      READ(1,*) (BLFPER(J), J = 1, NBLIFE)

      WRITE(3,*) ''
      WRITE(3,*) '                            ANALYSIS INPUT PARAMETERS'
      WRITE(3,*) '*****************************************************'
      
      READ(1,*) FAILURE
      WRITE(3,*) '      Probability of Failure Required =', FAILURE      

C   FIND OUT THE INDEX FOR THE REQUIRED PROBABILITY OF FAILURE IN THE PARAMETER TABLE      
      BLFPOS = 0
      
      Do 100 J = 1, NBLIFE
        IF (BLFPER(J) .LE. FAILURE) THEN
            BLFPOS = BLFPOS + 1
        ENDIF
  100 CONTINUE       
                     
C   READ TERRAIN HISTORY      
      DO 50 I = 1, NTERRAIN
        READ(2,*) SLOPE(I), DRAG(I), VSPD(I), BRAKE_V(I), BRAKECOUNT(I), 
     &            ACCEL(I)
   50 CONTINUE

C   READ CLUTCH PARAMETER TABLE FROM FILES
      DO 60 I = 1, 4
         INQUIRE (FILE = TABLENAME(I), EXIST = FTEST)
         IF (FTEST .EQV. .TRUE.) THEN
            OPEN (FILNUM(I), FILE = TABLENAME(I), STATUS = 'OLD')
            READ(FILNUM(I),*) T_FORCE(I), T_A(I)
            DO 65 J = 1, NBLIFE
               READ(FILNUM(I),*) CLUTCHWEARTABLE(I, J, 0:8)
   65       CONTINUE
            CLOSE (FILNUM(I))
         ELSE
            WRITE(3,*) 'ERROR: CANNOT OPEN FILE, ', TABLENAME(I), 
     &                ' DOES NOT EXIST'
            CALL TRMNAT
         ENDIF
   60 CONTINUE
     
C   READ BRAKE PARAMETER TABLE FROM FILES
      DO 70 I = 1, 5
         INQUIRE (FILE = BRAKETABLENAME(I), EXIST = FTEST)
         IF (FTEST .EQV. .TRUE.) THEN
            OPEN (BRAKEFILNUM(I), FILE =BRAKETABLENAME(I),STATUS ='OLD')
            READ(BRAKEFILNUM(I),*) BRAKEPADAREA_A(I)
            DO 75 J = 1, NBLIFE
               READ(BRAKEFILNUM(I),*) BRAKEWEARTABLE(I, J, 5:14)
   75       CONTINUE
            CLOSE (BRAKEFILNUM(I))
         ELSE
            WRITE(3,*) 'ERROR: CANNOT OPEN FILE, ', TABLENAME(I), 
     &                ' DOES NOT EXIST'
            CALL TRMNAT
         ENDIF
   70 CONTINUE
        
  900 FORMAT(15X,'GEAR      GEAR RATIO     GEAR EFFICIENCY')

  901 FORMAT(15X, I2, 10X, F6.3, 10X, F6.3)
  
  903 FORMAT(20X, E20.10, ' * X^^', I1, '  +')

  904 FORMAT(20X, E20.10, /, 30X, 'where X = Engine Torque')

  905 FORMAT(20X, E20.10, /, 30X, 'where X = Vehicle Speed')

      RETURN
      END
      
C************************************************************************
C   FUNCTION ENGTORQUECURVE1 CALCULATE THE ENGINE SPEED FROM TORQUE CURVE
C
C   DATE :      JANUARY 2013
C
C************************************************************************
      DOUBLE PRECISION FUNCTION ENGTORQUECURVE1(X)
            
      INTEGER   MAXPOWER

      PARAMETER (MAXPOWER = 8)
      
      INTEGER   I, POWER1

      DOUBLE PRECISION  COEF1(0:MAXPOWER)
      
      REAL  X, ENGFACTOR

      COMMON/CURVE1/   COEF1, POWER1

      COMMON/FACTOR/   ENGFACTOR
      
      X = X / ENGFACTOR

      ENGTORQUECURVE1 = 0.0
        
      DO 10 I = 0, POWER1
            ENGTORQUECURVE1 = ENGTORQUECURVE1 + COEF1(I) 
     &      * (X ** (POWER1-I))
   10 CONTINUE
      
      RETURN
      END

C************************************************************************
C   FUNCTION ENGTORQUECURVE2 CALCULATE THE ENGINE SPEED FROM TORQUE CURVE
C
C   DATE :      APRIL 2013
C
C************************************************************************
      DOUBLE PRECISION FUNCTION ENGTORQUECURVE2(X)
            
      INTEGER   MAXPOWER

      PARAMETER (MAXPOWER = 8)
      
      INTEGER   I, POWER2

      DOUBLE PRECISION  COEF2(0:MAXPOWER)
      
      REAL  X, ENGFACTOR

      COMMON/CURVE2/   COEF2, POWER2

      COMMON/FACTOR/   ENGFACTOR

      X = X / ENGFACTOR

      ENGTORQUECURVE2 = 0.0
        
      DO 10 I = 0, POWER2
            ENGTORQUECURVE2 = ENGTORQUECURVE2 + COEF2(I) 
     &      * (X ** (POWER2-I))
   10 CONTINUE
      
      RETURN
      END
      
C************************************************************************
C   FUNCTION ENGTORQUECURVE3 CALCULATE THE ENGINE SPEED FROM TORQUE CURVE
C
C   DATE :      APRIL 2013
C
C************************************************************************
      DOUBLE PRECISION FUNCTION ENGTORQUECURVE3(X)
            
      INTEGER   MAXPOWER

      PARAMETER (MAXPOWER = 8)
      
      INTEGER   I, POWER3

      DOUBLE PRECISION  COEF3(0:MAXPOWER)
      
      REAL  X, ENGFACTOR

      COMMON/CURVE3/   COEF3, POWER3

      COMMON/FACTOR/   ENGFACTOR

      X = X / ENGFACTOR

      ENGTORQUECURVE3 = 0.0
        
      DO 10 I = 0, POWER3
            ENGTORQUECURVE3 = ENGTORQUECURVE3 + COEF3(I) 
     &      * (X ** (POWER3-I))
   10 CONTINUE
      
      RETURN
      END

C************************************************************************
C   FUNCTION DLINERTIA CALCULATE THE DRIVELINE ADDED MASS (% TANK WEIGHT) 
C           FROM VEHICLE VELOCITY
C
C   DATE :      MAY 2013
C
C************************************************************************
      DOUBLE PRECISION FUNCTION DLINERTIA(X)
            
      INTEGER   MAXPOWER

      PARAMETER (MAXPOWER = 8)
      
      INTEGER   I, POWER4

      DOUBLE PRECISION  COEF4(0:MAXPOWER)
      
      REAL  X, DL_MINSPD

      COMMON/CURVE4/   COEF4, POWER4, DL_MINSPD

      DLINERTIA = 0.0
           
      IF ( X .LE. DL_MINSPD) THEN
        DLINERTIA = 20.0
      ELSE        
        DO 10 I = 0, POWER4
            DLINERTIA = DLINERTIA + COEF4(I) 
     &      * (X ** (POWER4-I))
   10   CONTINUE
      ENDIF
     
      DLINERTIA = DLINERTIA / 100.0
      
      RETURN
      END
      
C************************************************************************
C   SUBROUTINE SHIFTLOGIC PERFORM THE SHIFT LOGIC AND CALCULATE THE 
C       CUMULATIVE SN FOR EACH GEAR
C
C   DATE :      APRIL 2013
C
C************************************************************************
      SUBROUTINE SHIFTLOGIC (ENGSPDMAX, GEAR, GRATIO, GSHIFT, MAXGEAR,  
     &          SN, SHIFTTIME)
      
      INTEGER   MAXG, MAXTERRAIN

      REAL      PI, G_S, G_C
      
      PARAMETER (MAXG = 6, MAXTERRAIN = 20000)
      
      PARAMETER (PI = 3.1415926536, G_S = 32.174, G_C = 32.174 )
      
      INTEGER   GEAR, GSHIFT(0:MAXG), I, J, K, M, MAXGEAR
      
      REAL      GRATIO(0:MAXG), ENGSPDMAX, 
     &          RELSHIFTSPD, SHIFTTIME, SN(0:MAXG)

      DO WHILE (GEAR .NE. MAXGEAR) 
C   SHIFTING REQUIRED        
           
        IF (GEAR .LT. MAXGEAR) THEN
C   UPSHIFT REQUIRED (Sequential Shift) : 
C       LOWER GEAR CLUTCH: 
            GSHIFT(GEAR) = GSHIFT(GEAR) + 1
            RELSHIFTSPD = ENGSPDMAX * 
     &          ( GRATIO(GEAR + 1) / GRATIO(GEAR) - 1 )
            SN(GEAR) = SN(GEAR) 
     &          + SHIFTTIME * ABS( RELSHIFTSPD ) * PI / 30.0

C   UP GEAR:
            GEAR = GEAR + 1
                
C       HIGHER GEAR CLUTCH:
            IF (GEAR .GT. 1)    THEN
                GSHIFT(GEAR) = GSHIFT(GEAR) + 1
                RELSHIFTSPD = ENGSPDMAX * 
     &              ( GRATIO(GEAR) / GRATIO(GEAR - 1) - 1)
                SN(GEAR) = SN(GEAR) 
     &              + SHIFTTIME * ABS( RELSHIFTSPD ) * PI / 30.0
            ENDIF
                
        ELSE IF (GEAR .GT. MAXGEAR) THEN
C   DOWNSHIFT REQUIRED (Direct Shift):  
            M = GEAR - MAXGEAR
                
C   HIGHER GEAR CLUTCH:
            IF (GEAR .GT. 1)    THEN
                GSHIFT(GEAR) = GSHIFT(GEAR) + 1
                RELSHIFTSPD = ENGSPDMAX *
     &              ( GRATIO(GEAR) / GRATIO(GEAR - M) - 1)
                SN(GEAR) = SN(GEAR) 
     &              +  SHIFTTIME * ABS( RELSHIFTSPD ) * PI / 30.0
            ENDIF

C   DOWN GEAR:
            GEAR = GEAR - M
                
C   LOWER GEAR CLUTCH: 
            GSHIFT(GEAR) = GSHIFT(GEAR) + 1
            RELSHIFTSPD = ENGSPDMAX * 
     &          ( GRATIO(GEAR + M) / GRATIO(GEAR) - 1 )
            SN(GEAR) = SN(GEAR) 
     &          + SHIFTTIME * ABS( RELSHIFTSPD) * PI / 30.0
          
        ENDIF  
        
      END DO
      
      RETURN 
      END

C************************************************************************
C   SUBROUTINE DELTABRAKEENERGY CALCULATE THE DELTA ENERGY FOR BRAKE WEAR
C       CALCULATION
C
C   DATE :      MAY 2013
C
C************************************************************************
      REAL FUNCTION DELTABRAKEENERGY (ACCEL, ROTOR_OMEGA1, 
     &          ROTOR_OMEGA2, TANK_MASS, SLOPE, V1, V2)
      
      REAL      PI, G_S, G_C
            
      PARAMETER (PI = 3.1415926536, G_S = 32.174, G_C = 32.174 )
      
      INTEGER   I, J, K, M
      
      REAL      ACCEL, DLINERTIA, H2, 
     &          I_ROTOR, MASS1, MASS2, ROTOR_OMEGA1, ROTOR_OMEGA2, 
     &          SLOPE, STOPDIST, TANK_MASS, TANKMASS1, TANKMASS2,  
     &          V1, V2

      REAL      D_KE, D_KE_ROTOR, D_PE, KE_R_OMEGA1, KE_R_OMEGA2, 
     &          KE_V_V1, KE_V_V2

      COMMON/BRAKEENERGY/ I_ROTOR
 
C   CALCULATE ADJUSTED TANK MASS (in lbm) AT V1 AND V2 
      TANKMASS1 = TANK_MASS * (1.0 + DLINERTIA(V1) ) 
      TANKMASS2 = TANK_MASS * (1.0 + DLINERTIA(V2) ) 
              
C   CALCULATE STOPING DISTANCE (in ft)
      STOPDIST =  ABS( ( ( V2 * 5280.0 / 3600.0 ) ** 2 - 
     &             ( V1 * 5280.0 / 3600.0 ) ** 2 ) 
     &              / 2.0 / ( ACCEL * G_S ) )

C   CALCULATE DELTA VEHICLE KENETIC ENERGY (lbf-ft) FROM V1 TO V2
      KE_V_V1 = 0.5 / G_C * TANKMASS1 * 
     &              ( V1 * 5280.0 / 3600.0 ) ** 2.0
      KE_V_V2 = 0.5 / G_C * TANKMASS2 * 
     &              ( V2 * 5280.0 / 3600.0 ) ** 2.0
         
      D_KE = KE_V_V2 - KE_V_V1
      
C   CALCULATE DELTA ROTOR KENETIC ENERGY FROM ROTOR_OMEGA1 TO ROTOR_OMEGA2    
      KE_R_OMEGA1 = 0.5 * I_ROTOR 
     &          *  ( ROTOR_OMEGA1 * 2.0 * PI / 60.0 ) ** 2.0
      KE_R_OMEGA2 = 0.5 * I_ROTOR 
     &          *  ( ROTOR_OMEGA2 * 2.0 * PI / 60.0 ) ** 2.0      
      
      D_KE_ROTOR = KE_R_OMEGA2 - KE_R_OMEGA1
      
C   CALCULATE DELTA POTENTIAL ENERGY FROM V1 TO V2
      H2 = STOPDIST * SIN( SLOPE )
      
      D_PE = TANKMASS2 / G_C * G_S * H2
      
C   CALCULATE ENERGY DISSIPATED DUE TO BRAKING FROM V1 TO V2      
      DELTABRAKEENERGY = - ( D_KE + 2.0 * D_KE_ROTOR + D_PE )
      
      IF (DELTABRAKEENERGY .LT. 0.0) THEN
        DELTABRAKEENERGY = 0.0
      ENDIF
                                    
      RETURN 
      END
      
C************************************************************************
C   FUNCTION WEAR INTERPOLATE THE WEAR FRACTION FOR THE CLUTCH FROM PARAMETER MAP
C
C   DATE :      JANUARY 2013
C
C************************************************************************
      REAL FUNCTION WEAR(SN)
      
      INTEGER   MAXBLF

      PARAMETER (MAXBLF = 35)

      INTEGER   BLFPOS, I, J, K, SNPOS

      REAL      A, FORCE, T_A(4), T_FORCE(4), W(4, 2), 
     &          W_GEO(2), W_SN(4), CLUTCHWEARTABLE(4, MAXBLF, 0:8), SN

      COMMON/CLUTCHMAP/  A, FORCE, T_A, T_FORCE, CLUTCHWEARTABLE
      
      COMMON/MAP/      BLFPOS

      
C   FIND OUT THE INDEX FOR RANGE IN WHICH SN LIES
      SNPOS = 0
      
      DO 10, I = 0, 8
        IF ( ( 10 ** REAL(I) ) .LE. SN )  THEN
           SNPOS = I
        ENDIF
   10 CONTINUE     

C   LOOKUP PARAMETER TABLE
      DO 20, I = 1, 4
            W(I, 1) = CLUTCHWEARTABLE(I, BLFPOS, SNPOS)
            W(I, 2) = CLUTCHWEARTABLE(I, BLFPOS, SNPOS + 1)
   20 CONTINUE         

C   INTERPOLATE FOR NORMALIXED SLIDING DISTANCE
      DO 30, I = 1, 4
        W_SN(I) = ( W(I,2) - W(I,1) ) / 
     &            ( (10**REAL(SNPOS+1)) - (10**REAL(SNPOS)) ) *
     &            ( SN - (10**REAL(SNPOS)) ) + W(I,1)
   30 CONTINUE 
           
C   INTERPOLATE FOR CLUTCH GEOMETRIC PARAMETER
      W_GEO(1) = ( W_SN(2) - W_SN(1) ) / 
     &           ( T_A(2) - T_A(1) ) *
     &           ( A - T_A(1) ) + W_SN(1)

      W_GEO(2) = ( W_SN(4) - W_SN(3) ) / 
     &           ( T_A(4) - T_A(3) ) *
     &           ( A - T_A(3) ) + W_SN(3)

C   INTERPOLATE FOR NORMAL FORCE
      WEAR = ( W_GEO(2) - W_GEO(1) ) / 
     &       ( T_FORCE(3) - T_FORCE(1) ) *
     &       ( FORCE - T_FORCE(1) ) + W_GEO(1)

      RETURN
      END

C************************************************************************
C   FUNCTION BRAKEWEAR INTERPOLATE THE WEAR FRACTION FOR THE BRAKE FROM PARAMETER MAP
C
C   DATE :      MAY 2013
C
C************************************************************************
      REAL FUNCTION BRAKEWEAR(E)
      
      INTEGER   MAXBLF

      PARAMETER (MAXBLF = 35)

      INTEGER   AREA_POS, BLFPOS, ENERGY_POS, I, J, K

      REAL      BRAKEPADAREA, BRAKEPADAREA_A(5), 
     &          BRAKEWEARTABLE(5, MAXBLF, 5:14), 
     &          W(0:1, 0:1), W_E(0:1), LIMIT, E
      
      COMMON/BRAKEMAP/ BRAKEPADAREA, BRAKEPADAREA_A, BRAKEWEARTABLE
      
      COMMON/MAP/      BLFPOS
       
C   FIND OUT THE INDEX FOR RANGE IN WHICH ENERGY LIES
      DO 10, I = 5, 14
        LIMIT = 10**(REAL(I))
        IF (LIMIT .LE. E )  THEN
           ENERGY_POS = I
        ENDIF

   10 CONTINUE     
        
C   FIND OUT THE INDEX FOR RANGE IN WHICH BRAKE PAD AREA LIES
      DO 15, I = 1, 5
        IF (BRAKEPADAREA_A(I) .LE. BRAKEPADAREA )  THEN
           AREA_POS = I
        ENDIF
   15 CONTINUE     

C   LOOKUP PARAMETER TABLE
      DO 20, I = 0, 1
        DO 25, J = 0, 1
            W(I, J) = BRAKEWEARTABLE(AREA_POS + I, BLFPOS, 
     &                  ENERGY_POS + J )
   25   CONTINUE 
   20 CONTINUE 

C   INTERPOLATE FOR ENERGY     
      DO 30, I = 0, 1
        W_E(I) = ( W(I,1) - W(I,0) ) / 
     &      ( (10**REAL(ENERGY_POS+1)) - (10**REAL(ENERGY_POS))) *
     &      ( E - (10**REAL(ENERGY_POS)) ) + W(I,0) 
   30 CONTINUE 
        
C   INTERPOLATE FOR BRAKE PAD AREA     
      BRAKEWEAR = ( W_E( 1 )- W_E( 0 ) ) / 
     & ( BRAKEPADAREA_A( AREA_POS + 1 ) - BRAKEPADAREA_A( AREA_POS ) ) *
     & ( BRAKEPADAREA - BRAKEPADAREA_A( AREA_POS ) ) + W_E( 0 ) 

      RETURN
      END
      
C************************************************************************
C   SUBROUTINE BRAKEPADAREACAL CALCULATE BRAKE PAD AREA REQUIRED
C           TO PRODUCE MIN. BRAKE DECELERATION 
C
C   DATE :      MAY 2013
C
C************************************************************************
      SUBROUTINE BRAKEPADAREACAL( ACCELMAX, BRAKEPADMINAREA, 
     &          BRAKEPADPRESS, FDRATIO, MUI_BRAKE, 
     &          ROTORADIUS, TANK_MASS, TRACKDRAG, SPROCKETRADIUS )

      REAL      PI, G_S, G_C
            
      PARAMETER (PI = 3.1415926536, G_S = 32.174, G_C = 32.174 )

      REAL      ACCELMAX, BRAKEPADMINAREA, BRAKEPADPRESS,  
     &          FDRATIO, MUI_BRAKE, ROTORADIUS, TANK_MASS, TRACKDRAG, 
     &          SPROCKETRADIUS 
     
      REAL      FORCE_PAD, FORCE_ROTOR, FORCE_SPROCKET, TORQUE_ROTOR, 
     &          TORQUE_SPROCKET
      
C   Brake Force At Sprocket
      FORCE_SPROCKET = TANK_MASS * ( ABS(ACCELMAX) - TRACKDRAG )
        
C   Brake Torque At Sprocket
      TORQUE_SPROCKET = FORCE_SPROCKET * SPROCKETRADIUS
      
C   Brake Torque
      TORQUE_ROTOR = TORQUE_SPROCKET / FDRATIO
      
C   Brake Force at Effective Radius of Rotor
      FORCE_ROTOR = TORQUE_ROTOR / ( ROTORADIUS / 12.0 ) 
      
C   Max. Brake Force at Brake Pad      
      FORCE_PAD = FORCE_ROTOR / MUI_BRAKE
      
C   Min. Brake Pad Area Required (in^2)
      BRAKEPADMINAREA = FORCE_PAD / BRAKEPADPRESS 
                  
      RETURN
      END

C************************************************************************
C   SUBROUTINE BRAKEPERFORMANCE CALCULATE BRAKE DECELERATION CAPABILITY 
C           AND MAX. BRAKING DISTANCE FROM MAX VELOCITY 
C
C   DATE :      MAY 2013
C
C************************************************************************
      SUBROUTINE BRAKEPERFORMANCE(ACCEL_FAULT, BRAKE_D, BRAKEDIST_FAULT, 
     &          BRAKEENERGY, BRAKETORQUE, BRAKEWEARFRACTION, FDRATIO, 
     &          MAXFORCE_PAD, MUI_BRAKE, ROTORADIUS, SPROCKETRADIUS, 
     &          TANK_MASS, TOPSPD, TRACKDRAG)

      REAL      PI, G_S, G_C
            
      PARAMETER (PI = 3.1415926536, G_S = 32.174, G_C = 32.174 )

      REAL      ACCEL_FAULT, BRAKE_D, BRAKEDIST_FAULT,  
     &          BRAKETORQUE, BRAKEWEAR, BRAKEWEARFRACTION, FDRATIO, 
     &          FORCE_SPROCKET, MAXFORCE_PAD, MUI_BRAKE, ROTORADIUS, 
     &          SPROCKETRADIUS, TANK_MASS, TORQUE_SPROCKET, 
     &          TRACKDRAG, BRAKEENERGY
      
C   CALCULATE WEAR FRACTION/WEAR DAMAGE FOR BRAKE PAD
      IF (BRAKEENERGY .LT. 100000) THEN
        BRAKEWEARFRACTION = 0.0
      ELSE
        BRAKEWEARFRACTION = BRAKEWEAR(BRAKEENERGY)
      ENDIF
      
      IF (BRAKEWEARFRACTION .LE. 0.8) THEN
        BRAKE_D = 0.0
      ELSEIF (BRAKEWEARFRACTION .LE. 1.0) THEN
        BRAKE_D = 5.0 * (BRAKEWEARFRACTION - 0.8 )
      ELSE
        BRAKE_D = 1.0
      ENDIF    

      BRAKETORQUE = (1.0 - BRAKE_D) * MUI_BRAKE  
     &      * MAXFORCE_PAD * ( ROTORADIUS / 12.0 )

C   Brake Torque At Sprocket
      TORQUE_SPROCKET = BRAKETORQUE * FDRATIO 

C   Brake Force At Sprocket
      FORCE_SPROCKET = TORQUE_SPROCKET / SPROCKETRADIUS
     
C   BRAKE DECELERATION CAPABILITY
      ACCEL_FAULT = - (FORCE_SPROCKET / TANK_MASS + TRACKDRAG )

C   MAX. STOPING DISTANCE FROM MAX VELOCITY WITH MAX BRAKE DECELERATION
      BRAKEDIST_FAULT = -(TOPSPD * 1.466666667)** 2.0 / 2.0 
     &      / ( ACCEL_FAULT * G_C )

      RETURN
      END
