C************************************************************************
C   PROGRAM LCF CONTROLS THE FLOW OF LOGIC OF A LOW CYCLE FATIGUE ANALYSIS
C
C      DEVELOPED BY ANGELES CREST ENGINEERING INC
C
C   DATE:       OCTOBER 2012
C
C   SUBPROGRAMS:INFAGG, INPUT, PARDRW, HYPDRW, WORSTN, 
C                   TRMNAT, LIFCAL, INSORT, SORTM, EXPTCD
C
C   FILES:      1:LCFDAT-OLD; 2:COMPLCFDAT-OLD;    
C               3:LCFOUT-NEW; 8:IOUTPR-NEW; 9:LOWLIF-NEW;
C
C************************************************************************
      PROGRAM LCF

      IMPLICIT NONE
      
      LOGICAL   FILETEST3, FILETEST8, FILETEST9, FTEST

      INTEGER   MAXBLF, MAXDAT, MAXLIF, MAXM, MAXMM, MAXNGEOM, 
     &          MAXNLDHIS, MAXREG, MAXSGEOM

      PARAMETER (MAXBLF = 35, MAXDAT = 100, MAXLIF = 100000,
     &          MAXM = 20000, MAXMM = 100001, MAXNGEOM = 3, 
     &          MAXNLDHIS = 1, MAXREG = 3, MAXSGEOM = 11)

      COMMON    IOUT

      INTEGER   BLFPOS(MAXBLF), I, IOUT, J, K, L, M, MCOUNT, MID, MPROC, 
     &          N, NBLIFE, NGEOM, NHYPER, NLDHIS, NLIFE, NLIFET, NMED, 
     &          NPTS(MAXREG), NSYM, NTIME, NUMREG, P, Q,
     &          SGEOM(MAXNGEOM), VARY, ZROREG, FITFILNUM(11)

      DOUBLE PRECISION  RAND

      REAL      ALLM(MAXMM, MAXREG), BIGK(0:MAXREG), BIGK1, 
     &          BLFPER(MAXBLF), BLOCK, BZERO, EM, EPSL, EPSW, EPSMAX, 
     &          EPSMAX_A(MAXSGEOM, MAXSGEOM, MAXSGEOM), FACTR,  
     &          FIFTY, FTU, FTY, GEOMDATA(MAXNGEOM, MAXSGEOM),   
     &          GEOM(MAXNGEOM), INVLIFEL(MAXLIF), INVLIFEW(MAXLIF), 
     &          INVLIFLA(MAXSGEOM, MAXSGEOM, MAXSGEOM, MAXLIF), 
     &          INVLIFWA(MAXSGEOM, MAXSGEOM, MAXSGEOM, MAXLIF), 
     &          KRATIO, LAMGEOM(MAXNGEOM), LAMNEUB, LAMSTR, LAMDA,  
     &          LAMLD(MAXNLDHIS), LIFEL(MAXLIF), LIFEL_A(MAXSGEOM,  
     &          MAXSGEOM, MAXSGEOM, MAXLIF), LIFEW(MAXLIF),
     &          LIFEW_A(MAXSGEOM, MAXSGEOM, MAXSGEOM, MAXLIF), 
     &          LIFL, LIFW, LNA(0:MAXREG),  
     &          LNZ, LPHIM(0:MAXREG), MEDKB(0:MAXREG), MEDM(MAXREG),  
     &          MEDMB(0:MAXREG), MM(0:MAXREG), MU(MAXREG),  
     &          NBND(0:MAXREG), NF(MAXDAT, MAXREG), PGEOM(MAXNGEOM), 
     &          PHI, PSIG, PVAR,
     &          RAMCOEF, RAMEXPO, RANGEM(2, MAXREG), RESID(MAXDAT),   
     &          SBND(0:MAXREG), SIG(MAXREG), STR(MAXDAT, MAXREG), 
     &          STRHIS(MAXM), SZERO, TOTLIF, TRBIGK(0:MAXREG), 
     &          TRSBND(0:MAXREG), TRUNC, USTRHIS(MAXNLDHIS, MAXM), WEXP,
     &          Z

      CHARACTER*40 DESCGEOM(MAXNGEOM)
      
      CHARACTER*10 FITDATANAME(11)
      
      DATA (FITDATANAME(I), I = 1, 11) /
     &     'FITDATA1','FITDATA2','FITDATA3','FITDATA4', 'FITDATA5', 
     &     'FITDATA6','FITDATA7','FITDATA8','FITDATA9', 'FITDATA10',  
     &     'FITDATA11'/
     
      DATA (FITFILNUM(I), I = 1, 11) /
     &     21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31/

     
C ** OPEN INPUT AND OUTPUT FILES ==============================================
      OPEN (1,  FILE = 'LCFDAT', STATUS = 'OLD')
      OPEN (2,  FILE = 'COMPLCFDAT', STATUS = 'OLD')
      INQUIRE (FILE = 'LCFOUT', EXIST = FILETEST3)
      INQUIRE (FILE = 'IOUTPR', EXIST = FILETEST8)
      INQUIRE (FILE = 'LOWLIF', EXIST = FILETEST9)
      IF (FILETEST3 .EQV. .TRUE.) THEN
        OPEN (3,  FILE = 'LCFOUT', STATUS = 'OLD')
        CLOSE (UNIT=3, STATUS='DELETE')
      ENDIF  
      IF (FILETEST8 .EQV. .TRUE.) THEN
        OPEN (8,  FILE = 'IOUTPR', STATUS = 'OLD')
        CLOSE (UNIT=8, STATUS='DELETE')
      ENDIF  
      IF (FILETEST9 .EQV. .TRUE.) THEN  
        OPEN (9,  FILE = 'LOWLIF', STATUS = 'OLD')
        CLOSE (UNIT=9, STATUS='DELETE')
      ENDIF  
      
      OPEN (3,  FILE = 'LCFOUT', STATUS = 'NEW')
      OPEN (8,  FILE = 'IOUTPR', STATUS = 'NEW')
      OPEN (9,  FILE = 'LOWLIF', STATUS = 'NEW')
       
C===============================================================================
      CALL  INPUT   (BLFPER, BLOCK, DESCGEOM, EM, GEOMDATA, USTRHIS, 
     &   MPROC, NBLIFE, NGEOM, NHYPER, NLDHIS, NLIFE, NMED, NSYM, NTIME, 
     &        PGEOM, RAMCOEF, RAMEXPO, RAND, SGEOM, TRUNC, VARY, WEXP)

C   PERFORM THE INFORMATION AGGREGATION OF THE MATERIALS CHARACTERIZATION 
C       MODEL CALCULATIONS
      CALL INFAGG   (RANGEM, MU, SIG, NF, NPTS, SZERO, ZROREG, NUMREG,
     &              NBND, STR, FTU, FTY, VARY, MPROC, KRATIO, PVAR, 
     &              MEDMB, MEDKB, RESID)

      IF (MPROC .EQ. 1) THEN
        PSIG = SQRT (PVAR)
      ELSE
        PSIG = 0.0
      ENDIF
      
C   INITIALIZE VARIABLE
      DO 10 I = 1, MAXNGEOM
        GEOM(I) = 0.0
   10 CONTINUE       
   
      NLIFET = NHYPER * NLIFE
          
C   PARAMETER 1 LOOP    
      DO 500 M = 1, SGEOM(1)
            
C   PARAMETER 2 LOOP
        DO 510 N = 1, SGEOM(2)   

C   PARAMETER 3 LOOP
            DO 520 P = 1, SGEOM(3)           
                GEOM(1) =  GEOMDATA(1,M)
                WRITE (8,*) 'PARAMETER #1 ', DESCGEOM(1),'=', GEOM(1)
                WRITE (9,*) 'PARAMETER #1 ', DESCGEOM(1),'=', GEOM(1)

                IF (NGEOM .GT. 1)   THEN
                    GEOM(2) = GEOMDATA(2,N)
                    WRITE (8,*) 'PARAMETER #2 ', DESCGEOM(2),'=', 
     &                          GEOM(2)
                    WRITE (9,*) 'PARAMETER #2 ', DESCGEOM(2),'=',
     &                          GEOM(2)
                    IF (NGEOM .GT. 2)   THEN
                        GEOM(3) = GEOMDATA(3,P)
                        WRITE (8,*) 'PARAMETER #3 ', DESCGEOM(3),'=',
     &                              GEOM(3)
                        WRITE (9,*) 'PARAMETER #3 ', DESCGEOM(3),'=', 
     &                              GEOM(3)
                    ENDIF
                ENDIF        
                                
C   INITIALIZE VARIABLES
                MCOUNT = 0
                
                DO 35 K = 1, MAXLIF
                    LIFEW(K) = 1.0E+36
                    LIFEL(K) = 1.0E+36
   35           CONTINUE
      
                EPSMAX = 0.0
                
C ** OUTER LOOP -- THIS LOOP SAMPLES HYPER-PARAMETER SETS
                DO 150 K = 1, NHYPER
       
C ** CALL PEB TO PERFORM THE BOOTSTRAPPING ASPECT OF THE
C      MATERIALS CHARACTERIZATION MODEL CALCULATIONS
                    CALL PEB (NPTS, NUMREG, ZROREG, RAND, NBND, STR, 
     &                       MEDMB, MEDKB, RESID, BIGK, BZERO, MM, SBND)

C ** OBTAIN MATERIALS PROCESS VARIATION PARAMETERS IF DESIRED
                    CALL NORMGN (RAND, 0.0, PSIG, LNZ)

                    IF (MPROC .EQ. 1) THEN
                            Z = EXP (LNZ)
                    ELSE
                            KRATIO = 1.0
                            Z = 1.0
                            LNZ = 0.0
                    ENDIF

                    MCOUNT = MCOUNT + 1
         
                    DO 175 L = 1, NUMREG
                        ALLM(MCOUNT, L) = MM(L)
  175               CONTINUE
  
C ** INNER LOOP -- THIS LOOP GENERATES BLADE FAILURE TIMES
                    DO 200 I = 1, NLIFE

C ** INITILIZE S/N CURVE PARAMETERS
                        DO 225 L = 0, MAXREG
                            LNA(L) = 0.0
                            LPHIM(L) = 0.0
                            TRSBND(L) = 0.0
  225                   CONTINUE          
  
                        CALL PARDRW (LAMDA, LAMGEOM, LAMLD, LAMNEUB, 
     &                              LAMSTR, RAND)

                        CALL WORSTN (RAND, NSYM, BZERO, MM, EPSW, EPSL)

                        IF(VARY .EQ. 0) THEN
                            DO 220 L = ZROREG, NUMREG
                                MM(L) = MEDMB(L)
                                BIGK(L) = MEDKB(L)
  220                       CONTINUE
                            EPSW = 1.00
                            EPSL = 1.00
                        ENDIF
                                                
                        IF ((VARY .EQ. 0) .OR. (VARY .EQ. 4)) PHI = 1.0

C ** CALCULATE REGION DEPENDENT S/N CURVE PARAMETERS
                        FACTR = PHI * KRATIO * Z
            
                        DO 235 L = ZROREG, NUMREG
                            TRSBND(L) = FACTR * SBND(L)
                            TRBIGK(L) = BIGK(L)
  235                   CONTINUE
  
                        TRSBND(0) = SBND(0)
            
                        IF (ZROREG .EQ. 0) CALL KOMO (SZERO, BIGK, 
     &                      MM, NBND, TRSBND, TRBIGK, FACTR, NUMREG)

                        DO 250 L = ZROREG, NUMREG
                            LNA(L) = MM(L) * ALOG(TRBIGK(L))
                            LPHIM(L) = MM(L) * ALOG(PHI)
  250                   CONTINUE

C  PERFORM UNIT-STRESS-HISTORY TO STRESS-HISTORY TRANSFORMATION 
                        CALL STRAN (GEOM, NGEOM, NLDHIS, LAMGEOM, LAMLD,   
     &                           LAMSTR, NTIME, PGEOM, STRHIS, USTRHIS)


C ** CALL LIFCAL TO OBTAIN LCF LIFE
                        CALL LIFCAL (BLOCK, EM, EPSMAX, FTY,  
     &                      KRATIO, LAMNEUB, LNA, LNZ, LPHIM, MM,  
     &                      NUMREG, NTIME, RAMCOEF, RAMEXPO, SBND, 
     &                      STRHIS, SZERO, TOTLIF, TRSBND, TRUNC, WEXP, 
     &                      ZROREG)     
      
c                        LIFW = EPSW * TOTLIF * LAMDA
                        LIFL = EPSL * TOTLIF * LAMDA

                        IF (NLIFET .GE. 100) THEN
c                            CALL INSORT (LIFW, LIFEW, NLIFET)
                            CALL INSORT (LIFL, LIFEL, NLIFET)
                        ENDIF

C   INNER LOOP END
  200               CONTINUE    
C   OUTER LOOP END     
  150           CONTINUE  
  
C   CALCULATE DAMAGE
                DO 160 J = 1, NLIFET
                    INVLIFEL(J) = 1.0 / LIFEL(J)
c                    INVLIFEW(J) = 1.0 / LIFEW(J)
  160           CONTINUE
                
                write(8,*) 'EPSMAX =', EPSMAX          
             
                IF (NLIFET .GE. 100) THEN
C ** PRINT SORTED LIVES AND DAMAGE TO FILE LOWLIF
C                    DO 300 J = 1, (NLIFET)
C                    DO 300 J = 1, (NLIFET / 100)
C                        WRITE(9,*) J, FLOAT(J)/FLOAT(NLIFET), 
C     &                        LIFEW(J), LIFEL(J)
C     &                        , INVLIFEW(J), INVLIFEL(J)
C  300               CONTINUE

C ** INITIALIZE VARIABLE BLFPOS()
                    DO 325 J = 1, MAXBLF
                        BLFPOS(J) = 0
  325               CONTINUE

c                    FIFTY = 0.50E0

C ** PRINT EMPIRICAL BLIVES
                    WRITE(3,*) ''
                    WRITE(3,*) ''
                    WRITE(3,*) 'LIFE AND DAMAGE DATA FOR'
                    
                    WRITE(3,*) 'PARAMETER #1 ', DESCGEOM(1), '=', 
     &                          GEOM(1)
                    IF (NGEOM .GT. 1)   THEN
                        WRITE(3,*) 'PARAMETER #2 ', DESCGEOM(2), '=', 
     &                          GEOM(2)
                        IF (NGEOM .GT. 2)  THEN
                        WRITE(3,*) 'PARAMETER #3 ', DESCGEOM(3), '=', 
     &                          GEOM(3)
                        ENDIF                                       
                    ENDIF
                    
                    WRITE(3,925)

                    DO 350 J = 1, NBLIFE
                        BLFPOS(J) = NINT (BLFPER(J) * FLOAT (NLIFET))
c                        WRITE(3,926) BLFPER(J), LIFEW(BLFPOS(J)), 
c     &                          LIFEL(BLFPOS(J))
                        WRITE(3,926) BLFPER(J), LIFEL(BLFPOS(J)), 
     &                              INVLIFEL(BLFPOS(J))
  350               CONTINUE
c                    WRITE(3,926) FIFTY, LIFEW(NLIFET/2), LIFEL(NLIFET/2)
c                   WRITE(3,926) FIFTY, LIFEL(NLIFET/2), 
c     &                              INVLIFEL(NLIFET/2)
                    

C ** CALCULATE NORMAL MEDIAN CURVE IF DESIRED
                    IF(((VARY .EQ. 3) .AND. (NMED .EQ. 1)) 
     &                             .OR. (VARY .EQ. 4)) THEN
                        CALL SORTM (ALLM, NUMREG, MCOUNT)

                        MID = MCOUNT / 2
                        DO 400 L = 1, NUMREG
                            MEDM(L) = ALLM(MID,L)
  400                   CONTINUE

                        CALL EXPCTD (1, MEDM, NPTS, STR, NF, SZERO, 
     &                      NUMREG, ZROREG, NBND, BIGK1, BZERO)
                    ENDIF
                ENDIF
        
                EPSMAX_A(M, N, P) = EPSMAX

                DO 530 J = 1, NLIFET
c                    LIFEW_A(M, N, P, J) = LIFEW(J)
                    LIFEL_A(M, N, P, J) = LIFEL(J)
                    INVLIFLA(M, N, P, J) = INVLIFEL(J)
c                    INVLIFWA(M, N, P, J) = INVLIFEW(J)
  530           CONTINUE 
            
C   PARAMETER 3 LOOP END       
  520       CONTINUE                
C   PARAMETER 2 LOOP END   
  510   CONTINUE        
C   PARAMETER 1 LOOP END  
  500 CONTINUE    

C   WRITE DAMAGE FOR DATA FIT
c      DO 370 I = 1, 1
c         INQUIRE (FILE = FITDATANAME(1), EXIST = FTEST)
c         IF (FTEST .EQV. .TRUE.) THEN
c            OPEN (FITFILNUM(1), FILE = FITDATANAME(1), STATUS = 'OLD')
c            CLOSE(FITFILNUM(1), STATUS='DELETE')
c         ELSE
c            OPEN (FITFILNUM(1),  FILE = FITDATANAME(1), STATUS = 'NEW')
c            DO 360 J = 1, NLIFET
c                WRITE(FITFILNUM(1),*) INVLIFLA(1, 1, 1, J), 
c     &              INVLIFLA(2, 1, 1, J), INVLIFLA(3, 1, 1, J), 
c     &              INVLIFLA(4, 1, 1, J), INVLIFLA(5, 1, 1, J),  
c     &              INVLIFLA(6, 1, 1, J), INVLIFLA(7, 1, 1, J),  
c     &              INVLIFLA(8, 1, 1, J), INVLIFLA(9, 1, 1, J),
c     &              INVLIFLA(10, 1, 1, J), INVLIFLA(11, 1, 1, J)       
c  360       CONTINUE
c         ENDIF
c  370 CONTINUE


C  925 FORMAT(/2X,'B LIVES:        WEIBULL         LOGNORMAL',/)
  925 FORMAT(/2X,'B LIVES:        LIFE        DAMAGE',/)

  926 FORMAT(2X,F7.5,5X,E13.6,5X,E13.6)
    
      STOP

      END

C************************************************************************
C   SUBROUTINE LIFCAL CALCULATES THE LIFE GIVEN THE STRESS HISTORY 
C
C   DATE:   OCTOBER 2012
C
C   SUBPROGRAMS:RAINFLOW, GTLIFE, NEUBER
C       
C   INPUTS:     KRATIO, LNA,  LNZ, LPHIM, MM, NUMREG, NRNDS, 
C                   NTIME, SBND, STRHIS, SZERO, TRSBND,
C                   TRUNC, WEXP, ZROREG
C
C   OUTPUTS:    TOTLIF
C
C           LIST OF VARIABLES
C
C   IOUT        CONTROLS DUMP TO FILE IOUTPR
C   KRATIO      RATIO OF K*/K, CONSTANT OVER REGIONS AND COMPONENTS
C   LNA()       1-D ARRAY CONTAINING Ln(A) = Ln(BIGK)*MM FOR EACH REGION
C   LNZ         NORMAL(0,PVAR) GENERATED RANDOM VARIABLE
C   LPHIM()     1-D ARRAY CONTAINING Ln(PHI)*MM FOR EACH REGION
C   MAXM        MAXIMUM NUMBER OF POINTS ALLOWED IN TIME HISTORY
C   MAXREG      MAXIMUM NUMBER OF REGIONS ALLOWED
C   MM()        1-D ARRAY CONTAINING VALUES OF M FOR EACH REGION
C   NTIME       NUMBER OF POINTS IN STRESS-TIME HISTORY
C   NUMREG      NUMBER OF REGIONS OF INTEREST
C   TRUNC       VALUE USED TO FILTER OUT NOISE IN THE TIME HISTORY (%)
C   WEXP        WALKER EXPONENT
C   ZROREG      ZeRO REGion -- VALUES CHOSEN TO FACILITATE REGION DO LOOP
C                   BEGINNING VALUE -- 0 - ZERO REGION EXISTS, 1 - NO ZERO 
C                   REGION
C
C************************************************************************
      SUBROUTINE LIFCAL (BLOCK, EM, EPSMAX, FTY, KRATIO,   
     &                   LAMNEUB, LNA, LNZ, LPHIM, MM, NUMREG, NTIME, 
     &                   RAMCOEF, RAMEXPO, SBND, STRHIS, SZERO, TOTLIF,
     &                   TRSBND, TRUNC, WEXP, ZROREG)

      IMPLICIT NONE

      INTEGER  MAXM, MAXREG, MAXLIF

      PARAMETER (MAXM = 20000, MAXREG = 3, MAXLIF = 100000)

      COMMON   IOUT

      INTEGER  I, IOUT, NCYC, NTIME, NUMREG, ZROREG

      REAL     BLOCK, EM, EPSMAX, FTY, GTLIFE, KRATIO, INVLIF(MAXLIF), 
     &         LAMNEUB, LNA(0:MAXREG), LNZ, LPHIM(0:MAXREG), 
     &         LIFE(MAXLIF), MM(0:MAXREG), 
     &         RAMCOEF, RAMEXPO, SBND(0:MAXREG), 
     &         SEFFM(2,MAXM), SRANGE(MAXM), STRHIS(MAXM), SUMDAM, 
     &         SZERO, TRSBND(0:MAXREG), TOTLIF, TRUNC, WEXP

C   INITIALIZE THE DAMAGE
      SUMDAM = 0.0
           
C   FIND STRESS CYCLES
      CALL RAINFLOW (STRHIS, NTIME, TRUNC, SEFFM, NCYC)
     
C   DETERMINE EQUIVALENT STRAIN RANGE USING NEUBER'S RULE AND WALKER RELATION
      CALL NEUBER (EM, EPSMAX, FTY, NCYC, LAMNEUB, RAMCOEF, 
     &                    RAMEXPO, SEFFM, SRANGE, WEXP )

C   Calculate lives and damage fractions: LIFE(N) and INVLIF(N)
      DO 600 I=1, NCYC
        LIFE(I) = GTLIFE (SRANGE(I), MM, LNA, LPHIM, KRATIO, LNZ, 
     &                     SBND, ZROREG, NUMREG, SZERO) / BLOCK
 600  CONTINUE
      
      DO 650 I=1,NCYC
        INVLIF(I) = 1.0 / LIFE(I)
 650  CONTINUE
 
C  Miner's Rule -- sum the damage fractions
      DO 680 I=1,NCYC
        SUMDAM = SUMDAM + INVLIF(I)
  680 CONTINUE
      
C  calculate fatigue life 
      TOTLIF = 1.0 / SUMDAM

      RETURN
      END

C************************************************************************
C   SUBROUTINE INSORT PERFORMS AN INSERTION SORT FOR EACH LIFE CALCULATED
C
C   DATE:       OCTOBER 2012
C
C   VERSION:    2.1
C
C   SUBPROGRAMS:N/A
C
C   INPUTS:     NEWLIF, LIFE, NLIFET
C
C   OUTPUTS:    LIFE
C
C           LIST OF VARIABLES
C
C   I           CONTROLS DO LOOP FOR INSERTION
C   IOUT        OUTPUT DUMP CONTROLLER
C   LIFE()      1-D ARRAY CONTAINING TAIL VALUES OF THE LIVES GENERATED BY THE
C                   PFM TO BE SORTED
C   MAXLIF      MAXIMUM NUMBER OF FATIGUE LIVES ALLOWED FOR BETA, THETA, ALPHA,
C                   CALCULATION
C   NEWLIF      LIFE VALUE TO BE INSERTED INTO LIFE()
C   NLIFET      TOTAL NUMBER OF LIVES CALCULATED BY PFM
C   NUM         NUMBER OF LIFE VALUES IN LIFE()
C   PLACE       POSITION WHERE NEWLIF IS TO BE INSERTED INTO LIFE()
C   TEMP()      1-D ARRAY CONTAINING VALUES OF LIFE() TO BE SHIFTED UPON 
C                   INSERTION OF NEWLIF
C
C************************************************************************
      SUBROUTINE INSORT (NEWLIF, LIFE, NLIFET)

      IMPLICIT NONE

      INTEGER  MAXLIF

      PARAMETER (MAXLIF = 100000)

      COMMON   IOUT

      INTEGER  I, IOUT, NLIFET, NUM, PLACE

      REAL     LIFE(MAXLIF), NEWLIF, TEMP(MAXLIF)

c      NUM = NLIFET / 2.0
       NUM = NLIFET

C   FIND POSITION IN LIFE() FOR NEWLIF
c      IF (NEWLIF .GT. LIFE(NUM)) GOTO 400

      DO 100 I = 1, NUM
         IF (NEWLIF .LT. LIFE(I)) THEN
            PLACE = I
            GOTO 110
         ENDIF
  100 CONTINUE
  110 CONTINUE

C   STORE VALUES OF LIFE() TO BE SHIFTED DUE TO NEWLIF INSERTION IN TEMP()
      DO 200 I = (PLACE + 1), NUM
         TEMP(I) = LIFE(I-1)
  200 CONTINUE

C   INSERT NEWLIF
      LIFE(PLACE) = NEWLIF

C   SHIFT VALUES OF LIFE() FOLLOWING NEWLIF
      DO 300 I = (PLACE + 1), NUM
         LIFE(I) = TEMP(I)
  300 CONTINUE

C   IF NEWLIF IS LARGER THAN ALL LIVES IN LIFE() THEN RETURN
c  400 CONTINUE

      RETURN
      END
            
C************************************************************************
C   SUBROUTINE TRMNAT HANDLES THE TERMINATION OF THE PROGRAM RUN WHEN
C       ONE OF THE PROGRAM'S ASSUMPTIONS HAVE BEEN VIOLATED
C
C************************************************************************
      SUBROUTINE TRMNAT

      WRITE(8,*) 'PROGRAM EXECUTION TERMINATED'
      STOP
      END  

C************************************************************************
C   SUBROUTINE 'SORTM' SORTS THE ARRAY, ALLM(), FROM LOWEST TO HIGHEST 
C       M FOR EACH REGION
C
C   DATE:       OCTOBER 2012
C
C   VERSION:    MATCHR V7, V7.1, V8, V8.1, V8.2, V8.3, V8.4, V8.5
C               MATGRM V4, V4.1, V4.2, V4.3, V4.4, V4.5
C
C   SUBPROGRAMS:N/A
C
C   INPUTS:     ALLM, NUMREG, NUM
C
C   OUTPUTS:    ALLM
C
C           LIST OF VARIABLES
C
C   ALLM()      2-D ARRAY CONTAINING VALUES TO BE SORTED FOR EACH REGION
C   I           CONTROLS INSERTION POINTER
C   INC         SORT INCREMENT VARIABLE
C   INORDR      FLAG TO INDICATE WHETHER SORT IS FINISHED
C   IOUT        OUTPUT DUMP CONTROLLER
C   L           CONTROLS DO LOOP FOR EACH REGION
C   MAXMM       MAXIMUM NUMBER OF M'S TO BE SORTED
C   MAXREG      MAXIMUM NUMBER OF REGIONS ALLOWED
C   NUM         NUMBER OF ELEMENTS IN ALLM() TO BE SORTED
C   NUMREG      NUMBER OF REGIONS OF INTEREST
C   TEMP        TEMPORARY SORTING VARIABLE
C
C************************************************************************
      SUBROUTINE SORTM (ALLM, NUMREG, NUM)

      IMPLICIT NONE

      COMMON  IOUT

      INTEGER I, INC, IOUT, L, MAXMM, MAXREG, NUM, NUMREG

      PARAMETER (MAXMM = 100001, MAXREG = 3)

      LOGICAL INORDR

      REAL    ALLM(MAXMM, MAXREG), TEMP

      DO 400 L = 1, NUMREG

    5    INC = NUM
   10    IF (INC .GT. 1) THEN
            INC = INC / 2
   20       INORDR = .TRUE.

            DO 300 I = 1, (NUM - INC)
               IF (ALLM(I,L) .GT. ALLM(I + INC, L)) THEN
                  TEMP = ALLM(I,L)
                  ALLM(I,L) = ALLM(I + INC, L)
                  ALLM(I + INC, L) = TEMP
                  INORDR = .FALSE.
               ENDIF
  300       CONTINUE

            IF (.NOT. INORDR) GOTO 20
            GOTO 10
         ENDIF

  400 CONTINUE

      RETURN
      END

C**********************************************************************
C   SUBROUTINE RAINFLOW CALCULATES THE STRESS CYCLES GIVEN STRESS HISTORY 
C
C   DATE:       OCTOBER 2012
C
C   SUBPROGRAMS:N/A
C
C   INPUTS:     SEFF, M, TRUNC
C
C   OUTPUTS:    SEFFM
C
C           LIST OF VARIABLES
C
C   CHKFT       DUMMY VARIABLE USED TO PRINT OUT RAINFLOW RESULT
C   E()         HOLDING ARRAY USED TO FIND CYCLES DURING RAINFLOW ANALYSIS
C   I,J,K       COUNTERS FOR VARIOUS DO LOOPS
C   IOUT        OUTPUT DUMP CONTROLLER
C   INDEX(MAXM) INTERMEDIATE CALCULATION ARRAYS USED DURING FILTERING
C   JMAX        INDEX (LOCATION) OF SEFMAX IN SEFF()
C   M           TOTAL NUMBER OF STRESS HISTORY POINTS
C   MAXM        MAXIMUM NUMBER OF POINTS ALLOWED IN STRESS-TIME HISTORY 
C                ARRAYS
C   N           NUMBER OF CYCLES FOUND DURING RAINFLOW ANALYSIS
C   NEWTOT      TOTAL NUMBER OF EFFECTIVE STRESS VALUES AFTER FILTERING
C   S(NEWTOT)   FILTERED EFFECTIVE STRAINS
C   SEFF(M)     STRESS HISTORY BEFORE FILTERING/RAINFLOW
C   SEFFM(2,N)  EFFECTIVE STRESS AFTER RESEQUENCING/FILTERING/RAINFLOW
C                  SEFFM(1,I) = sigma max,eff,i
C                  SEFFM(2,I) = sigma min,eff,i
C   SEFMAX      LARGEST EFFECTIVE STRESS
C   SP(M+1)     RESEQUENCED EFFECTIVE STRESSES; # OF PTS = M+1
C   TEST1(MAXM) INTERMEDIATE CALCULATION ARRAYS USED DURING FILTERING
C   TEST2(MAXM) INTERMEDIATE CALCULATION ARRAYS USED DURING FILTERING
C   TRUNC       VALUE USED TO FILTER OUT NOISE
C
C**********************************************************************
      SUBROUTINE RAINFLOW (SEFF, M, TRUNC, SEFFM, N)

      IMPLICIT NONE

      COMMON  IOUT

      INTEGER  MAXM

      PARAMETER (MAXM = 20000)

      INTEGER  I, INDEX(MAXM), IOUT, J, JMAX, K, M, N, NEWM, NEWTOT

      REAL    CHKFT, SEFF(MAXM), SEFFM(2, MAXM), TRUNC, E(MAXM), 
     &        S(MAXM), SEFMAX, SP(MAXM), TEST1(MAXM), TEST2(MAXM)

C  INITIALIZE ARRAYS
      DO 50 I = 1, MAXM
         SP(I) = 0.0
         S(I) = 0.0
         E(I) = 0.0
         SEFFM(1,I) = 0.0
         SEFFM(2,I) = 0.0
         INDEX(I) = 0
         TEST1(I) = 0.0
         TEST2(I) = 0.0
   50 CONTINUE

C=================== B E G I N    R E S E Q U E N C E ===================
C   RESEQUENCE effective strains (needed for rainflow analysis);
C       largest effective strain is placed at beginning and end of SP(M+1)
      J = 0
      SP(1) = SEFF(1)
      DO 100 I = 2,M
        IF((SEFF(I) - SEFF(I-1)) .NE. 0.0)  THEN
         J = J + 1
         SP(J) = SEFF(I)
        ENDIF
 100  CONTINUE
      NEWM = J
      DO 150 I =1,NEWM
        SEFF(I) = SP(I)
 150  CONTINUE

C   find SEFMAX, the largest sigma,eff, and JMAX, its location within SEFF(M)
      SEFMAX = -1.0E+20
      DO 200 I=1,NEWM
         IF ( SEFF(I) .GT. SEFMAX ) THEN
            SEFMAX = SEFF(I)
            JMAX = I
         ENDIF
 200  CONTINUE

C   assign all points from JMAX out, to the beginning of SP()
      DO 210 I = 1, NEWM-JMAX+1
         J = JMAX-1 + I
         SP(I) = SEFF(J)
 210  CONTINUE

C   assign points before JMAX to the end of SP()
      J = 0
      DO 220 I = NEWM-JMAX+2, NEWM
         J = J + 1
         SP(I) = SEFF(J)
 220  CONTINUE
      SP(NEWM+1) = SEFF(JMAX)

C==================== E N D    R E S Q U E N C E ===================
C===================== B E G I N    F I L T E R ====================
C   FILTER the resequenced effective strains, leaving only peaks and valleys
C       (excursions larger than TRUNC are deleted during rainflow counting) in
C       S(NEWTOT), where NEWTOT is the new number of points
      DO 300 I = 2, NEWM
         TEST1(I) = SP(I-1) - SP(I)
         TEST2(I) = TEST1(I) * (SP(I) - SP(I+1))
  300 CONTINUE

      K = 1
      INDEX(1) = 1

      DO 310 I = 2, NEWM
         IF ((TEST1(I) .NE. 0) .AND. (TEST2(I) .LT. 0)) THEN
            K = K + 1
            INDEX(K) = I
         ENDIF
  310 CONTINUE

      NEWTOT = K + 1
      INDEX(NEWTOT) = NEWM + 1

      DO 320 I = 1, NEWTOT
         K = INDEX(I)
         S(I) = SP(K)
  320 CONTINUE

C========================== E N D    F I L T E R =========================
C====================== B E G I N    R A I N F L O W ======================
C   RAINFLOW ANALYSIS to identify cycles within effective strain data, S(NEWTOT);
C       places each cycle's max and min values into SEFFM(2,N)
C
C       counters: I counts # of cycles found, J counts how many S()'s counted,
C           K accumulates unmatched points
      I = 0
      J = 0
      K = 0
 400  CONTINUE
      J = J+1
      K = K+1
C   check J to avoid reading beyond end of filtered strain data
      IF ( J .GT. NEWTOT ) GOTO 499

C   read strain point into a holding array to be checked for cycles
      E(K) = S(J)
 410  IF ( K .LT. 3 ) GOTO 400
      IF ( ABS( E(K) - E(K-1) ) .LT. ABS( E(K-1) - E(K-2) ) ) GOTO 400
C    if not, then a cycle has been found, but we need to check for truncation
      IF (ABS (E(K-1) - E(K-2)) .GT. TRUNC) THEN
C        cycle is large enough to save
         I = I+1
         SEFFM(1,I) = AMAX1( E(K-1), E(K-2) )
         SEFFM(2,I) = AMIN1( E(K-1), E(K-2) )
      ENDIF
C   discard points K-1 and K-2, and decrement the counter of unmatched points
      E(K-2) = E(K)
      K = K-2
C   return for more counting
      GOTO 410

  499 CONTINUE
C   N equals the final number of cycles found
      N = I
        
      RETURN
      END
      
C**********************************************************************
C   FUNCTION GTLIFE CALCULATES THE CYCLES TO FAILURE FOR A PARTICULAR STRESS
C       BASED UPON THE MATERIALS CHARACTERIZATION S/N EQUATION
C
C   DATE:       OCTOBER 2012
C
C   VERSION:    MATCHR V8.3, V8.4, V8.5 -- FOR USE WITH PFM'S  
C
C   SUBPROGRAMS:N/A
C
C   INPUTS:     S, MM, LNA, LPHIM, KRATIO, LNZ, SBND, ZROREG, NUMREG, SZERO
C
C   OUTPUTS:    GTLIFE
C
C           LIST OF VARIABLES
C
C   GETLIF      VALUE TO BE ASSIGNED TO GTLIFE -- CYCLES TO FAILURE FOR
C                   THE REQUIRED STRESS LEVEL
C   IOUT        OUTPUT DUMP CONTROLLER
C   KRATIO      RATIO OF K*/K, CONSTANT OVER REGIONS AND COMPONENTS
C   L           CONTROLS DO LOOP FOR EACH REGION
C   LNA()       1-D ARRAY CONTAINING VALUES OF Ln(A) = M Ln K FOR EACH REGION
C   LNZ         NORMAL(0,PVAR) GENERATED RANDOM VARIATE
C   LPHIM()     1-D ARRAY CONTAINING VALUES OF M Ln PHI FOR EACH REGION WHERE 
C                   PHI IS A WEIBULL(BETAo, ETAo) GENERATED RANDOM VARIATE
C   MAXREG      MAXIMUM NUMBER OF REGIONS ALLOWED
C   MM()        1-D ARRAY CONTAINING SELECTED VALUES OF M FOR EACH REGION
C   NUMREG      NUMBER OF REGIONS OF INTEREST
C   S           VALUE OF STRESS/STRAIN FOR WHICH A VALUE OF LIFE (CYCLES TO 
C                   FAILURE) IS REQUIRED
C   SBND()      1-D ARRAY CONTAINING THE STRESS/STRAIN VALUES ( R = -1.0) 
C                   CORRESPONDING TO THE "LIFE BOUNDARY" VALUES FOR EACH REGION 
C                   CONTAINED IN NBND()
C   SZERO       STRESS TENSILE POINT, So
C   TEMP        TEMPORARY VARIABLE USED TO PREVENT ARITHMETIC UNDER AND OVER 
C                   FLOWS
C   ZROREG      ZeRO REGion -- VALUES CHOSEN TO FACILITATE REGION DO LOOP 
C                   BEGINNING VALUE -- 0 - ZERO REGION EXISTS, 1 - NO REGION
C
C**********************************************************************
      REAL FUNCTION GTLIFE (S, MM, LNA, LPHIM, KRATIO, LNZ, SBND, 
     &                      ZROREG, NUMREG, SZERO)

      IMPLICIT NONE

      INTEGER IOUT, L, MAXREG, NUMREG, ZROREG

      PARAMETER (MAXREG = 3)

      COMMON  IOUT

      REAL    GETLIF, KRATIO, LNA(0:MAXREG), LNZ, LPHIM(0:MAXREG), 
     &        MM(0:MAXREG), S, SBND(0:MAXREG), SZERO, TEMP
   
      GETLIF = 0.0

C  CALCULATE CYCLES TO FAILURE  
      IF ((S .GE. SZERO) .AND. (ZROREG .EQ. 0)) THEN
         GETLIF = 1.0
      ELSE
         DO 100 L = ZROREG, NUMREG
            IF (S .GT. SBND(L)) THEN
               TEMP = LNA(L) + LPHIM(L) + MM(L) * ( - ALOG(S) 
     &                    +  ALOG (KRATIO) + LNZ) 
               IF (TEMP .GT. 86.0) THEN
                  TEMP = 86.0
               ENDIF
               GETLIF = EXP (TEMP)
               GOTO 150
            ENDIF
  100    CONTINUE
      ENDIF
  150 CONTINUE

      GTLIFE = GETLIF

      RETURN
      END        

C***********************************************************************
C   SUBROUTINE PARDRW PERFORMS THE RANDOM LIFE DRIVER PARAMETER DRAWS IN 
C       THE INNER LOOP
C
C   DATE :      OCTOBER 2012
C
C************************************************************************
      SUBROUTINE PARDRW (LAMDA, LAMGEOM, LAMLD, LAMNEUB, 
     &                   LAMSTR, RAND)

      IMPLICIT NONE

      INTEGER   I, IOUT, MAXNGEOM, MAXNLDHIS
      
      PARAMETER (MAXNGEOM = 3, MAXNLDHIS = 1)

      DOUBLE PRECISION  RAND

      REAL      DUM, LAMGEOM(MAXNGEOM), 
     &          LAMGEOMA(MAXNGEOM), LAMGEOMB(MAXNGEOM), 
     &          LAMNEUB, LAMNEUBA, LAMNEUBB, 
     &          LAMSTR, LAMSTRA, LAMSTRB, LAMDA, LAMDAA, LAMDAB,
     &          LAMLD(MAXNLDHIS), LAMLDA(MAXNLDHIS), LAMLDB(MAXNLDHIS)

      COMMON/DRIVRS/    LAMGEOMA,   LAMGEOMB, 
     &                  LAMLDA,     LAMLDB,
     &                  LAMNEUBA,   LAMNEUBB, 
     &                  LAMSTRA,    LAMSTRB, 
     &                  LAMDAA,     LAMDAB 
     
      COMMON    IOUT

C   GEOMETRIC PARAMETERS DRAW
      DO 10 I = 1, MAXNGEOM
        CALL PRYRV (RAND, LAMGEOMA(I),LAMGEOMB(I), 
     &               0, 1, LAMGEOM(I), DUM)
   10 CONTINUE
   
C   LOAD DRAW
      DO 20 I = 1, MAXNLDHIS
        CALL PRYRV (RAND, LAMLDA(I), LAMLDB(I), 0, 1, LAMLD(I), DUM)
   20 CONTINUE     
    
C   MATERIAL DRAW   
      CALL PRYRV (RAND, LAMNEUBA, LAMNEUBB, 0, 1, LAMNEUB, DUM)
      CALL PRYRV (RAND, LAMSTRA, LAMSTRB, LAMDAA, LAMDAB, LAMSTR, LAMDA)
      
      LAMDA = EXP (LAMDA)
          
      RETURN
      END
     
C************************************************************************
C   SUBROUTINE INPUT READS IN THE DATA AND ECHOES IT 
C
C   DATE :      OCTOBER 2012
C
C************************************************************************
      SUBROUTINE INPUT (BLFPER, BLOCK, DESCGEOM, EM, GEOMDATA, USTRHIS,   
     &  MPROC, NBLIFE, NGEOM, NHYPER, NLDHIS, NLIFE, NMED, NSYM, NTIME, 
     &          PGEOM, RAMCOEF,RAMEXPO, RAND, SGEOM, TRUNC, VARY, WEXP)
 
      IMPLICIT NONE

      INTEGER   MAXBLF, MAXM, MAXNGEOM, MAXNLDHIS, MAXSGEOM

      PARAMETER (MAXBLF = 35, MAXM = 20000, MAXNGEOM = 3, 
     &           MAXNLDHIS = 1, MAXSGEOM = 11)

      INTEGER   FILNUM(4), I, IOUT, J, MPROC, 
     &          NBLIFE, NGEOM, NHYPER, NLDHIS, NLIFE, NMED, 
     &          NSYM, NTIME, SGEOM(MAXNGEOM), VARY

      DOUBLE PRECISION  RAND

      REAL      BLFPER(MAXBLF), BLOCK, EM, GEOMDATA(MAXNGEOM, MAXSGEOM), 
     &          MAXGEOM(MAXNGEOM), MINGEOM(MAXNGEOM), LAMDAA, LAMDAB, 
     &          LAMGEOMA(MAXNGEOM), LAMGEOMB(MAXNGEOM),
     &          LAMLDA(MAXNLDHIS), LAMLDB(MAXNLDHIS), 
     &          LAMNEUBA, LAMNEUBB, LAMSTRA, LAMSTRB, PGEOM(MAXNGEOM),
     &          RAMCOEF, RAMEXPO, TRUNC, USTRHIS(MAXNLDHIS, MAXM), WEXP
     
      CHARACTER*8 LDNAME(4)
      
      CHARACTER*40 DESCGEOM(MAXNGEOM)

      COMMON/DRIVRS/    LAMGEOMA,   LAMGEOMB, 
     &                  LAMLDA,     LAMLDB,
     &                  LAMNEUBA,   LAMNEUBB, 
     &                  LAMSTRA,    LAMSTRB, 
     &                  LAMDAA,     LAMDAB 
     
      COMMON    IOUT
      
      LOGICAL   FTEST
   
      DATA (LDNAME(I), I = 1, 4) /
     &     'STRHIS1','STRHIS2','STRHIS3','STRHIS4'/
     
      DATA (FILNUM(I), I = 1, 4) /
     &     11, 12, 13, 14 /

      READ(1,*) RAND
      WRITE(8,*) '                         RANDOM NUMBER SEED =', RAND
      READ(1,*) NLIFE
      WRITE(8,*) '                            INNER LOOP SIZE =', NLIFE
      READ(1,*) NHYPER
      WRITE(8,*) '                            OUTER LOOP SIZE =', NHYPER
      READ(1,*) NSYM
      WRITE(8,*) '                            SYMMETRY NUMBER =', NSYM
      READ(1,*) VARY
      WRITE(8,*) '          TYPE OF MATERIAL VARIATION DESIRED '
      WRITE(8,*) '                  (0 - NONE, 4 - BOOTSTRAP) =', VARY
      READ(1,*) NMED
      WRITE(8,*) '      NORMAL MEDIAN CURVE (0 - NO, 1 - YES) =', NMED
      READ(1,*) MPROC
      WRITE(8,*) '        MATERIALS PROCESS VARIATION DESIRED'
      WRITE(8,*) '                          (0 - NO, 1 - YES) =', MPROC

      IF (VARY .NE. 0 .AND. VARY .NE. 4) THEN
         WRITE(8,*) 'ERROR:  INVALID TYPE OF S/N VARIATION DESIRED'
         CALL TRMNAT
      ENDIF
      
      IF ((NMED .NE. 0) .AND. (NMED .NE. 1)) THEN
         WRITE(8,*) 'ERROR:  INVALID RESPONSE TO NORMAL MEDIAN ',
     &              'CURVE QUESTION'
         CALL TRMNAT
      ENDIF

      IF ((MPROC .LT. 0) .OR. (MPROC .GT. 1)) THEN
         WRITE(8,*) 'ERROR:  INVALID TYPE OF MATERIALS PROCESS ',
     &              'VARIATION DESIRED'
         CALL TRMNAT
      ENDIF

      READ(1,*) NBLIFE
      IF (NBLIFE .GT. 0) READ(1,*) (BLFPER(J), J = 1, NBLIFE)

      READ(2,*) NLDHIS,  NTIME, BLOCK
      WRITE(8,*) '                    NUMBER OF LOAD HISTORYS =', NLDHIS
      WRITE(8,*) '    NUMBER OF POINTS IN STRESS-TIME HISTORY =', NTIME
      WRITE(8,*) '    NUMBER OF BLOCKS OF STRESS-TIME HISTORY' 
      WRITE(8,*) '                            IN MISSION LIFE =', BLOCK
      
      IF (NLDHIS .GT. MAXNLDHIS)THEN
        WRITE(8,*) 'ERROR: TOO MANY LOAD HISTORYS'
        CALL TRMNAT
      ENDIF
      
      IF (NTIME .GT. MAXM) THEN
        WRITE(8,*) 'ERROR:  LOAD TIME HISTORY TOO LARGE'
        CALL TRMNAT
      ENDIF
   
      DO 15 I = 1, NLDHIS
        READ(2,*)   LAMLDA(I), LAMLDB(I)
   15 CONTINUE

C ** READ COMPONENT DATA 
      READ(2,*) NGEOM
      WRITE(8,*) '             NUMBER OF PARAMETERS =', NGEOM
      
      IF (NGEOM .GT. MAXNGEOM)THEN
        WRITE(8,*) 'ERROR: TOO MANY PARAMETERS'
        CALL TRMNAT
      ENDIF
      
C   INITIALIZE SGEOM, PGEOM
      DO 11 I = 1, MAXNGEOM
        SGEOM(I) = 1
        PGEOM(I) = 0
   11 CONTINUE
            
      DO 10 I = 1, NGEOM
        READ(2,*)   DESCGEOM(I), PGEOM(I), MINGEOM(I), MAXGEOM(I), 
     &              LAMGEOMA(I), LAMGEOMB(I), SGEOM(I)
        IF ((SGEOM(I) .EQ. 1) .AND. 
     &           (MAXGEOM(I) .NE. MINGEOM(I))) THEN
            WRITE(8,*) 'ERROR: SGEOM =1 WHEN MAXGEOM = MINGEOM'
            CALL TRMNAT
        ENDIF
        IF (SGEOM(I) .GT. MAXSGEOM) THEN
            WRITE(8,*) 'ERROR: TOO MANY SEGMENTS FOR PARAMETER ', NGEOM
            CALL TRMNAT
        ELSE IF (SGEOM(I) .LE. 0) THEN
            WRITE(8,*) 'ERROR: NEGATIVE SEGMENT FOR PARAMETER ', NGEOM
            CALL TRMNAT
        ENDIF
   10 CONTINUE
   
C CALCULATE INDIVIDUAL POINT FOR EACH PARAMETERS    
      DO 60 I = 1, NGEOM
        DO 70 J = 1, SGEOM(I)
            IF ( MAXGEOM(I) .EQ. MINGEOM(I) ) THEN
                GEOMDATA(I, J) = MAXGEOM(I)
            ELSE
                GEOMDATA(I, J) = (J-1) * ( MAXGEOM(I) - MINGEOM(I) )
     &                  / (SGEOM(I)-1) + MINGEOM(I)
            ENDIF
   70   CONTINUE                    
   60 CONTINUE
      
      
      READ(1,*) LAMNEUBA, LAMNEUBB, 
     &          LAMSTRA, LAMSTRB, 
     &          LAMDAA, LAMDAB 
      READ(1,*) TRUNC
      
      READ(2,*) EM, RAMCOEF, RAMEXPO, WEXP
            
C  READ LOAD TIME HISTORY FROM FILES
      DO 40 I = 1, NLDHIS
         INQUIRE (FILE = LDNAME(I), EXIST = FTEST)
         IF (FTEST .EQV. .TRUE.) THEN
            OPEN (FILNUM(I), FILE = LDNAME(I), STATUS = 'OLD')
            DO 35 J = 1, NTIME
               READ(FILNUM(I),*) USTRHIS(I, J)
   35       CONTINUE
            CLOSE (FILNUM(I))
         ELSE
            WRITE(8,*) 'ERROR: CANNOT OPEN FILE, ', LDNAME(I), 
     &                ' DOES NOT EXIST'
            CALL TRMNAT
         ENDIF
   40 CONTINUE
            
C   ECHO DATA TO ELFOUT
      WRITE(3,900)
      
      DO 45 I = 1, NGEOM
        WRITE(3, 901) I, DESCGEOM(I), MINGEOM(I),MAXGEOM(I), 
     &                   LAMGEOMA(I), LAMGEOMB(I), SGEOM(I)
   45 CONTINUE
   
      DO 160 I = 1, NGEOM
        WRITE(3, *), ' PARAMETER #',I, DESCGEOM(I)
        DO 170 J = 1, SGEOM(I)
            WRITE(3, *), GEOMDATA(I, J)  
  170   CONTINUE                    
  160 CONTINUE
        
      DO 50 I = 1, NLDHIS
        WRITE(3,903) I, LAMLDA(I), LAMLDB(I), NTIME
   50 CONTINUE
            
      WRITE(3,904)  LAMNEUBA, LAMNEUBB, 
     &              LAMSTRA, LAMSTRB 
     
      WRITE(3,905) EXP(LAMDAA), EXP(LAMDAB)
      
      WRITE(3,906) EM, RAMCOEF, RAMEXPO, TRUNC, WEXP
     
C   FORMAT STATEMENTS TO ECHO INPUT DATA TO LCFOUT
  900 FORMAT(28X,'ANGELES CREST ENGINEERING INC',//,
     &       21X,'LOW CYCLE FATIGUE',//
     &       36X,'INPUT DATA',
     &       ///,14X,'DRIVERS',28X,'PARAMETER DISTRIBUTIONS',//)

  901 FORMAT( 2X, 'PARAMETER #', I1, 5X, A40/,
     &        2X, 'LOWER BOUND = ', F9.5, 10X, 'UPPER BOUND = ', F9.5,/,
     &        2X, 'UNCERTAINTY  ',21X,   'U(',F9.5,',',F9.5,')',/,
     &        2X, 'NUMBER OF PARAMETER POINTS = 'I2,/)

  903 FORMAT(//,2X, 'LOAD HISTORY #', I1,/,
     &          2X  'UNCERTAINTY', 21X, 'U(',F9.5,',',F9.5,')',/,
     &          2X, 'NUMBER OF DATA POINTS = ' I4,/)
     
  904 FORMAT(//, 2X,'NEUBER RULE UNCERTAINTY',25X,
     &       'U(',F8.5,',',F9.5,')',
     &       //,2X,'STRESS ANALYSIS UNCERTAINTY',21X,
     &       'U(',F8.5,',',F9.5,')')

  905 FORMAT(/,2X,'DAMAGE MODEL ACCURACY',27X,
     &       'U(ln',F8.5,', ln',F8.5,')')

  906 FORMAT(////,20X,'LOAD STRESS INPUT',
     &       //,2X,'YOUNGS MODULUS',36X,E11.5,
     &       //,2X,'RAMBERG OSGOOD COEFFICIENT',24X,F6.4,
     &       //,2X,'RAMBERG OSGOOD EXPONENT',24X,F6.0,
     &       //,2X,'LOAD HISTORY NOISE FILTER (%)',21X,F7.5,
     &       //,2X,'WALKER EXPONENT',34X,F5.2,//)
     
      RETURN
      END

C***********************************************************************************************
C   NEUBER USES NEUBERS RULE AND THE RAMBERG OSGOOD STRESS-STRAIN CURVE TO CALCULATE TOTAL STRAIN
C       WALKER RELATION IS USED PERFORM MEAN CORRECTION
C
C   DATE:       OUTOBER 2012
C
C   INPUTS:     EM, FTY, NCYC, LAMNEUB,RAMCOEF, RAMEXPO, SEFFM, WEXP
C
C   OUTPUTS:    EPSMAX, SRANGE
C
C***********************************************************************************************
      SUBROUTINE NEUBER (EM, EPSMAX, FTY, NCYC, LAMNEUB,RAMCOEF, 
     &                    RAMEXPO, SEFFM, SRANGE, WEXP)

      IMPLICIT NONE

      INTEGER MAXM
      
      PARAMETER (MAXM = 20000) 

      COMMON IOUT

      INTEGER  I, IOUT, NCYC

     
      REAL  DELEPS, DELSIG,EM, EPSA, EPSB, EPSMAX, FTY, LAMNEUB, 
     &      RAMCOEF, RAMEXPO, RATIO, SEFFM(2,MAXM), SIGMA, 
     &      SRANGE(MAXM), WEXP

C  MAXIMUM STRAIN FROM NEUBER ANALYSIS
      DO 200 I=1, NCYC
         IF (SEFFM(1,I) .GT. 0.75*FTY) THEN
            DELSIG = SEFFM(1,I)/100.0
            SIGMA = DELSIG
            RATIO = 0.00      
            DO 100 WHILE (RATIO .LT. 1.0)
               EPSA = SIGMA/EM + RAMCOEF*(SIGMA/FTY)**RAMEXPO
               EPSB = LAMNEUB*(SEFFM(1,I)**2)/(SIGMA*EM)
               SIGMA = SIGMA + DELSIG
               RATIO = EPSA/EPSB 
 100        CONTINUE
            EPSB = SIGMA/EM
         ELSE
            EPSB = SEFFM(1,I)/EM
            
         ENDIF
         DELEPS = (SEFFM(1,I)-SEFFM(2,I))/EM
         IF(EPSB .GT. EPSMAX) EPSMAX = EPSB

C  WALKER RULE MEAN CORRECTION ONLY ON POSITIVE MEAN STRAIN
         IF (EPSMAX .GT. 0.0) THEN
            SRANGE(I) = DELEPS
     &               * (DELEPS / (2.0 * EPSMAX)) 
     &               ** (WEXP - 1.0)
         ELSE
            SRANGE(I) = DELEPS           
         ENDIF
     
 200  CONTINUE
      
      RETURN
      END
     
C************************************************************************ 
C   SUBROUTINE STRAN PERFORMS THE UNIT STRESS TO STRESS TRANSFORMATION 
C
C   DATE:       OCTOBER 2012
C************************************************************************ 
      SUBROUTINE STRAN (GEOM, NGEOM, NLDHIS, LAMGEOM, LAMLD,  
     &                  LAMSTR, NTIME, PGEOM, STRHIS, USTRHIS)
 
      INTEGER  MAXM, MAXNGEOM, MAXNLDHIS
 
      PARAMETER (MAXM = 20000, MAXNGEOM = 3, MAXNLDHIS = 1) 
 
      COMMON  IOUT 
 
      INTEGER  I, IOUT, J, K, NGEOM, NLDHIS, NTIME 
 
      REAL     GEOM(MAXNGEOM), LAMGEOM(MAXNGEOM), LAMLD(MAXNLDHIS), 
     &         LAMSTR, PGEOM(MAXNGEOM), STRHIS(MAXM), 
     &         USTRHIS(MAXNLDHIS,MAXM)
 
C   DERIVE STRESS HISTORY FROM UNIT STRESS HISTORY
      DO 100 I = 1, NTIME
        STRHIS(I) = USTRHIS(1, I) * ( LAMLD(1) * LAMSTR ) 
        DO 200 J = 1, NGEOM
            STRHIS(I) = STRHIS(I) * 
     &                  ( ( GEOM(J) * LAMGEOM(J) ) ** PGEOM(J) )
  200   CONTINUE      
  100 CONTINUE   
       
      RETURN  
      
      END 
