C************************************************************************
C   PROGRAM GEAR_SR PREPARE INPUT DATA FILES 
C           FOR STRESS RUPTURE ANALYSIS
C
C      DEVELOPED BY ANGELES CREST ENGINEERING INC
C
C   DATE:       NOVEMBER 2012
C
C   SUBPROGRAMS:INPUT, OUTPUT, STRAN, TRMNAT
C
C   FILES:      1:GEARSRDAT-OLD; 2:COMPSRDAT-NEW;    
C
C************************************************************************
      PROGRAM GEAR_SR

      IMPLICIT NONE
      
      LOGICAL   FILETEST2, FILETEST8

      INTEGER   MAXDAT, MAXM, MAXNGEOM, MAXNLDHIS, MAXSET, MAXSGEOM

      PARAMETER (MAXDAT = 100, MAXM = 20000, MAXNGEOM = 3, 
     &          MAXNLDHIS = 1,  MAXSET = 5, MAXSGEOM = 11)
            
      INTEGER   I, NGEOM, NLDHIS, NTIME, SGEOM(MAXNGEOM)

      REAL      MAXGEOM(MAXNGEOM), MINGEOM(MAXNGEOM), 
     &          LAMGEOMA(MAXNGEOM), LAMGEOMB(MAXNGEOM),
     &          LAMLDA(MAXNLDHIS), LAMLDB(MAXNLDHIS), 
     &          LDHIS(MAXNLDHIS, MAXM), PGEOM(MAXNGEOM), PSIG, FTMEAN
      
      CHARACTER*40 DESCGEOM(MAXNGEOM)

      CHARACTER*40  DESCRP(0:MAXSET)    

      REAL      PI
                
      PARAMETER (PI = 3.1415926536)

C ** OPEN INPUT AND OUTPUT FILES ==============================================
      OPEN (1,  FILE = 'GEARSRDAT', STATUS = 'OLD')
      INQUIRE (FILE = 'COMPSRDAT', EXIST = FILETEST2)
      INQUIRE (FILE = 'GEAROUT', EXIST = FILETEST8)

      IF (FILETEST2 .EQV. .TRUE.) THEN
        OPEN (2,  FILE = 'COMPSRDAT', STATUS = 'OLD')
        CLOSE (UNIT=2, STATUS='DELETE')
      ENDIF  

      IF (FILETEST8 .EQV. .TRUE.) THEN
        OPEN (8,  FILE = 'GEAROUT', STATUS = 'OLD')
        CLOSE (UNIT=8, STATUS='DELETE')
      ENDIF  

      OPEN (2,  FILE = 'COMPSRDAT', STATUS = 'NEW')
      OPEN (8,  FILE = 'GEAROUT', STATUS = 'NEW')

C READ DATA FROM GEARSRDAT
      CALL INPUT (NGEOM, NLDHIS, NTIME, SGEOM, MAXGEOM, MINGEOM,
     &          LAMGEOMA, LAMGEOMB, LAMLDA, LAMLDB, LDHIS, PGEOM, PSIG, 
     &          FTMEAN, DESCGEOM, DESCRP)
      
c     WRITE DATA TO COMPSRDAT     
      CALL OUTPUT (NGEOM, NLDHIS, NTIME, SGEOM, MAXGEOM, MINGEOM,
     &          LAMGEOMA, LAMGEOMB, LAMLDA, LAMLDB, PGEOM, PSIG, 
     &          FTMEAN, DESCGEOM, DESCRP)
           
      CALL STRAN(LDHIS, NLDHIS, NTIME, PGEOM) 
      
      STOP 
      
      END

C************************************************************************
C   SUBROUTINE INPUT READS IN THE DATA FROM GEARSRDAT
C
C   DATE :      OCTOBER 2012
C
C************************************************************************
      SUBROUTINE INPUT (NGEOM, NLDHIS, NTIME, SGEOM, MAXGEOM, MINGEOM,
     &          LAMGEOMA, LAMGEOMB, LAMLDA, LAMLDB, LDHIS, PGEOM, PSIG, 
     &          FTMEAN, DESCGEOM, DESCRP)
 
      INTEGER   MAXDAT, MAXM, MAXNGEOM, MAXNLDHIS, MAXREG, MAXSGEOM, 
     &          MAXSET

      PARAMETER (MAXDAT = 100, MAXM = 20000, MAXNGEOM = 3, 
     &          MAXNLDHIS = 1, MAXREG = 3, MAXSET = 5, MAXSGEOM = 11)

      INTEGER   FILNUM(4), I, IOUT, J, M, 
     &          NGEOM, NLDHIS, NTIME, SGEOM(MAXNGEOM)

      REAL      MAXGEOM(MAXNGEOM), MINGEOM(MAXNGEOM),  
     &          LAMGEOMA(MAXNGEOM), LAMGEOMB(MAXNGEOM),
     &          LAMLDA(MAXNLDHIS), LAMLDB(MAXNLDHIS), 
     &          LDHIS(MAXNLDHIS, MAXM), PGEOM(MAXNGEOM), PSIG, FTMEAN
     
      CHARACTER*6   LDNAME(6)
      
      CHARACTER*40  DESCGEOM(MAXNGEOM)

      CHARACTER*40  DESCRP(0:MAXSET)
       
      COMMON    IOUT
      
      LOGICAL   FTEST
   
      DATA (LDNAME(I), I = 1, 4) /
     &     'LDHIS1','LDHIS2','LDHIS3','LDHIS4'/
     
      DATA (FILNUM(I), I = 1, 4) /
     &     11, 12, 13, 14 /

      READ(1,*) NLDHIS,  NTIME
      
      IF (NLDHIS .GT. MAXNLDHIS)THEN
        WRITE(8,*) 'ERROR: TOO MANY LOAD HISTORYS'
        CALL TRMNAT
      ENDIF
      
      IF (NTIME .GT. MAXM) THEN
        WRITE(8,*) 'ERROR:  LOAD TIME HISTORY TOO LARGE'
        CALL TRMNAT
      ENDIF
               
      DO 15 I = 1, NLDHIS
        READ(1,*)   LAMLDA(I), LAMLDB(I)
   15 CONTINUE

C ** READ COMPONENT DATA 
      READ(1,*) NGEOM
      
      IF (NGEOM .GT. MAXNGEOM)THEN
        WRITE(8,*) 'ERROR: TOO MANY PARAMETERS'
        CALL TRMNAT
      ENDIF
      
C   INITIALIZE SGEOM, PGEOM
      DO 11 I = 1, MAXNGEOM
        SGEOM(I) = 1
        PGEOM(I) = 0
   11 CONTINUE
            
      DO 10 I = 1, NGEOM
        READ(1,*)   DESCGEOM(I), PGEOM(I), MINGEOM(I), MAXGEOM(I), 
     &              LAMGEOMA(I), LAMGEOMB(I), SGEOM(I)
        IF ((SGEOM(I) .EQ. 1) .AND. 
     &           (MAXGEOM(I) .NE. MINGEOM(I))) THEN
            WRITE(8,*) 'ERROR: SGEOM =1 WHEN MAXGEOM = MINGEOM'
            CALL TRMNAT
        ENDIF
        IF (SGEOM(I) .GT. MAXSGEOM) THEN
            WRITE(8,*) 'ERROR: TOO MANY SEGMENTS FOR PARAMETER ', NGEOM
            CALL TRMNAT
        ELSE IF (SGEOM(I) .LE. 0) THEN
            WRITE(8,*) 'ERROR: NEGATIVE SEGMENT FOR PARAMETER ', NGEOM
            CALL TRMNAT
        ENDIF
   10 CONTINUE   
          
C  READ LOAD TIME HISTORY FROM FILES
      DO 40 I = 1, NLDHIS
         INQUIRE (FILE = LDNAME(I), EXIST = FTEST)
         IF (FTEST .EQV. .TRUE.) THEN
            OPEN (FILNUM(I), FILE = LDNAME(I), STATUS = 'OLD')
            DO 35 J = 1, NTIME
               READ(FILNUM(I),*) LDHIS(I, J)
   35       CONTINUE
            CLOSE (FILNUM(I))
         ELSE
            WRITE(8,*) 'ERROR: CANNOT OPEN FILE, ', LDNAME(I), 
     &                ' DOES NOT EXIST'
            CALL TRMNAT
         ENDIF
   40 CONTINUE
      
C   INPUT DATA ON SPECIFIC MATERIAL
      READ(1,*) DESCRP(0), FTMEAN, PSIG
     
      RETURN
      END

C************************************************************************
C   SUBROUTINE OUTPUT CREATE THE DATAFILE COMPSRDAT FOR STRESSRUPTURE INPUT
C
C   DATE :      OCTOBER 2012
C
C************************************************************************
      SUBROUTINE OUTPUT (NGEOM, NLDHIS, NTIME, SGEOM, MAXGEOM, 
     &          MINGEOM, LAMGEOMA, LAMGEOMB, LAMLDA, LAMLDB, PGEOM, 
     &          PSIG, FTMEAN, DESCGEOM, DESCRP)
 
      INTEGER   MAXDAT, MAXM, MAXNGEOM, MAXNLDHIS, MAXREG, MAXSGEOM, 
     &          MAXSET

      PARAMETER (MAXDAT = 100, MAXM = 20000, MAXNGEOM = 3, 
     &          MAXNLDHIS = 1, MAXREG = 3, MAXSET = 5, MAXSGEOM = 11)

      INTEGER   FILNUM(4), I, IOUT, J, M,
     &          NGEOM, NLDHIS, NTIME, SGEOM(MAXNGEOM)

      REAL      MAXGEOM(MAXNGEOM), MINGEOM(MAXNGEOM),  
     &          LAMGEOMA(MAXNGEOM), LAMGEOMB(MAXNGEOM),
     &          LAMLDA(MAXNLDHIS), LAMLDB(MAXNLDHIS), PGEOM(MAXNGEOM), 
     &          PSIG, FTMEAN
     
      CHARACTER*6   LDNAME(6)
      
      CHARACTER*40  DESCGEOM(MAXNGEOM)

      CHARACTER*40  DESCRP(0:MAXSET)
       
      COMMON    IOUT
      
      WRITE(2,*) NLDHIS, NTIME
         
      DO 15 I = 1, NLDHIS
        WRITE(2,*)  LAMLDA(I), LAMLDB(I)
   15 CONTINUE

C ** WRITE COMPONENT DATA 
      WRITE(2,*) NGEOM
                  
      DO 10 I = 1, NGEOM
        WRITE(2,*)  '''',DESCGEOM(I), '''',PGEOM(I)
        WRITE(2,*)  MINGEOM(I), MAXGEOM(I), 
     &              LAMGEOMA(I), LAMGEOMB(I), SGEOM(I)
   10 CONTINUE   
          
  
C   OUPUT DATA ON SPECIFIC MATERIAL
      WRITE(2,*) '''',DESCRP(0),''''
      WRITE(2,*) FTMEAN, PSIG
     
      RETURN
      END
      
C************************************************************************ 
C   SUBROUTINE STRAN PERFORMS THE LOAD TO UNIT PARAMETER STRESS 
C       TRANSFORMATION AND CREATE THE UNIT PARAMETER STRESS DAT FILE
C
C   DATE:       DECEMBER 2012
C************************************************************************ 
      SUBROUTINE STRAN (LDHIS, NLDHIS, NTIME)
 
      INTEGER   FILNUM(4), MAXM, MAXNGEOM, MAXNLDHIS
 
      PARAMETER (MAXM = 20000, MAXNGEOM = 3, MAXNLDHIS = 1) 
 
      COMMON    IOUT 
 
      INTEGER   I, IOUT, J, K, NGEOM, NLDHIS, NTIME 
 
      REAL      LDHIS(MAXNLDHIS,MAXM), PGEOM(MAXNGEOM), 
     &          USTRHIS(MAXNLDHIS,MAXM)
      
      CHARACTER*8 LDNAME(4)

      LOGICAL   FTEST
 
      DATA (LDNAME(I), I = 1, 4) /
     &     'STRHIS1','STRHIS2','STRHIS3','STRHIS4'/
     
      DATA (FILNUM(I), I = 1, 4) /
     &     21, 22, 23, 24 /
     
C   DERIVE VON MISES STRESS HISTORY FROM LOAD HISTORY FOR GEAR 
C      EQUATION (11-15) from Shigley Page 400
C     * 12.0 to convert from ft-lb to in-lb
C   / 1000 to convert from psi to ksi. 
      
      DO 100 I = 1, NTIME
        USTRHIS(1,I) = 2 * LDHIS(1,I) * 12.0 / 1000.0   
  100 CONTINUE     

C  WRITE UNIT PARAMETER STRESS TIME HISTORY TO FILES
      DO 40 I = 1, NLDHIS
         INQUIRE (FILE = LDNAME(I), EXIST = FTEST)
         IF (FTEST .EQV. .TRUE.) THEN
            OPEN (FILNUM(I),  FILE = LDNAME(I), STATUS = 'OLD')
            CLOSE (UNIT=FILNUM(I), STATUS='DELETE')
         ENDIF
         
         OPEN (FILNUM(I), FILE = LDNAME(I), STATUS = 'NEW')
            DO 35 J = 1, NTIME
               WRITE (FILNUM(I),*) USTRHIS(I, J)
   35       CONTINUE
         CLOSE (FILNUM(I))
   40 CONTINUE
          
      RETURN  
      
      END 
      
C************************************************************************
C   SUBROUTINE TRMNAT HANDLES THE TERMINATION OF THE PROGRAM RUN WHEN
C       ONE OF THE PROGRAM'S ASSUMPTIONS HAVE BEEN VIOLATED
C
C************************************************************************
      SUBROUTINE TRMNAT

      WRITE(8,*) 'PROGRAM EXECUTION TERMINATED'
      STOP
      END  
