C************************************************************************
C   PROGRAM STRESSRUPTURE CONTROLS THE FLOW OF LOGIC OF A STRESS RUPTURE ANALYSIS
C
C      DEVELOPED BY ANGELES CREST ENGINEERING INC
C
C   DATE:       OCTOBER 2012
C
C   SUBPROGRAMS:INFAGG, INPUT, PARDRW, HYPDRW, WORSTN, 
C                   TRMNAT, LIFCAL, INSORT, SORTM, EXPTCD
C
C   FILES:      1:SRDAT-OLD; 2:COMPSRDAT-OLD;    
C               3:SROUT-NEW; 8:IOUTPR-NEW; 
C
C************************************************************************
      PROGRAM STRESSRUPTURE

      IMPLICIT NONE
      
      LOGICAL   FILETEST3, FILETEST8, FTEST

      INTEGER   MAXBLF, MAXDAT, MAXM, MAXNGEOM, MAXNLDHIS, MAXLIF, 
     &          MAXSGEOM

      PARAMETER (MAXBLF = 35, MAXDAT = 100, MAXM = 20000, MAXNGEOM = 3, 
     &          MAXNLDHIS = 1, MAXSGEOM = 11, MAXLIF = 100000)

      COMMON    IOUT

      INTEGER   BLFPOS(MAXBLF), I, IOUT, J, K, L, M, N, 
     &          NBLIFE, NGEOM, NLDHIS, NLIFE, NTIME, P, Q, 
     &          SGEOM(MAXNGEOM), FITFILNUM(11)

      DOUBLE PRECISION  RAND

      REAL      BLFPER(MAXBLF), FOS(MAXLIF), FOSL(MAXLIF), 
     &          FOS_A(MAXSGEOM, MAXSGEOM, MAXSGEOM, MAXLIF),
     &          FTMEAN, GEOMDATA(MAXNGEOM, MAXSGEOM),  
     &          GEOM(MAXNGEOM), MAXSTRESS, LAMGEOM(MAXNGEOM), LAMSTR,  
     &          LAMLD(MAXNLDHIS), PGEOM(MAXNGEOM), 
     &          PSIG, STRENGTH, USTRHIS(MAXNLDHIS, MAXM)


      CHARACTER*40 DESCGEOM(MAXNGEOM)
      
      CHARACTER*40 DESCRP

      CHARACTER*10 FITDATANAME(11)
      
      DATA (FITDATANAME(I), I = 1, 11) /
     &     'FITDATA1','FITDATA2','FITDATA3','FITDATA4', 'FITDATA5', 
     &     'FITDATA6','FITDATA7','FITDATA8','FITDATA9', 'FITDATA10',  
     &     'FITDATA11'/
     
      DATA (FITFILNUM(I), I = 1, 11) /
     &     21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31/
     
     
C ** OPEN INPUT AND OUTPUT FILES ==============================================
      OPEN (1,  FILE = 'SRDAT', STATUS = 'OLD')
      OPEN (2,  FILE = 'COMPSRDAT', STATUS = 'OLD')
      INQUIRE (FILE = 'SROUT', EXIST = FILETEST3)
      INQUIRE (FILE = 'IOUTPR', EXIST = FILETEST8)
      
      IF (FILETEST3 .EQV. .TRUE.) THEN
        OPEN (3,  FILE = 'SROUT', STATUS = 'OLD')
        CLOSE (UNIT=3, STATUS='DELETE')
      ENDIF  
      IF (FILETEST8 .EQV. .TRUE.) THEN
        OPEN (8,  FILE = 'IOUTPR', STATUS = 'OLD')
        CLOSE (UNIT=8, STATUS='DELETE')
      ENDIF  
      
      OPEN (3,  FILE = 'SROUT', STATUS = 'NEW')
      OPEN (8,  FILE = 'IOUTPR', STATUS = 'NEW')
       
C===============================================================================
      CALL  INPUT   (DESCGEOM, BLFPER, FTMEAN, GEOMDATA, NBLIFE, NGEOM, 
     &          NLDHIS, NLIFE, NTIME, PGEOM, PSIG, RAND, SGEOM, USTRHIS)

C   INITIALIZE VARIABLE
      DO 10 I = 1, MAXNGEOM
        GEOM(I) = 0.0
   10 CONTINUE                    
             
C   PARAMETER 1 LOOP    
      DO 500 M = 1, SGEOM(1)
            
C   PARAMETER 2 LOOP
        DO 510 N = 1, SGEOM(2)   

C   PARAMETER 3 LOOP
            DO 520 P = 1, SGEOM(3)           
                GEOM(1) =  GEOMDATA(1,M)

                IF (NGEOM .GT. 1)   THEN
                    GEOM(2) = GEOMDATA(2,N)
                    IF (NGEOM .GT. 2)   THEN
                        GEOM(3) = GEOMDATA(3,P)
                    ENDIF
                ENDIF        
                                
C   INITIALIZE VARIABLES
                DO 20 J = 1, NLIFE
                    FOS(J) = -1.0E+36
                    FOSL(J) = -1.0E+36
   20           CONTINUE     
                    
C   NLIFE LOOP  
                DO 200 I = 1, NLIFE

C   INITIALIZE MAXSTRESS
                    MAXSTRESS = 0.0

C   PERFORM DRIVER DRAW
                    CALL PARDRW (NGEOM, NLDHIS, LAMGEOM, LAMLD, LAMSTR, 
     &                              RAND)
     
C   PERFORM UNIT-STRESS-HISTORY TO STRESS-HISTORY TRANSFORMATION
C       AND FIND THE MAX STRESS 
                    CALL STRAN (GEOM, NGEOM, NLDHIS, LAMGEOM, LAMLD,   
     &                        LAMSTR, NTIME, PGEOM, MAXSTRESS, USTRHIS)

C   OBTAIN STRENGTH 
                    CALL NORMGN (RAND, FTMEAN, PSIG, STRENGTH) 
    
C   CALCULATE MAX FOS FOR STRESS-HISTORY
                    FOS(I) = STRENGTH / MAXSTRESS
C   SORT FOS
                    CALL INSORT (FOS(I), FOSL, NLIFE)
                  
C   NLIFE LOOP END
  200           CONTINUE    
  
                DO 530 J = 1, NLIFE
                    FOS_A(M, N, P, J) = FOSL(J)
  530           CONTINUE 

C ** PRINT EMPIRICAL PROBABLILITY OF FAILURE FOR EACH FOS
                WRITE(3,*) ''
                WRITE(3,*) ''
                WRITE(3,*) 'STRESS RUPTURE DATA FOR'
                    
                WRITE(3,*) 'PARAMETER #1 ', DESCGEOM(1), '=', 
     &                          GEOM(1)
                IF (NGEOM .GT. 1)   THEN
                    WRITE(3,*) 'PARAMETER #2 ', DESCGEOM(2), '=', 
     &                          GEOM(2)
                    IF (NGEOM .GT. 2)  THEN
                        WRITE(3,*) 'PARAMETER #3 ', DESCGEOM(3), '=', 
     &                          GEOM(3)
                    ENDIF                                       
                ENDIF     

                WRITE(3,925)


C ** INITIALIZE VARIABLE BLFPOS()
                    DO 325 J = 1, MAXBLF
                        BLFPOS(J) = 0
  325               CONTINUE

                    DO 350 J = 1, NBLIFE
                        BLFPOS(J) = NINT (BLFPER(J) * FLOAT (NLIFE))
                        WRITE(3,926) BLFPER(J), FOSL(BLFPOS(J))

  350               CONTINUE

C   PARAMETER 3 LOOP END       
  520       CONTINUE                
C   PARAMETER 2 LOOP END   
  510   CONTINUE        
C   PARAMETER 1 LOOP END  
  500 CONTINUE    

C   WRITE DAMAGE FOR DATA FIT
c      DO 370 I = 1, 5
c         INQUIRE (FILE = FITDATANAME(I), EXIST = FTEST)
c         IF (FTEST .EQV. .TRUE.) THEN
c            OPEN (FITFILNUM(I), FILE = FITDATANAME(I), STATUS = 'OLD')
c            CLOSE(FITFILNUM(I), STATUS='DELETE')
c         ENDIF
c         
c         OPEN (FITFILNUM(I),  FILE = FITDATANAME(I), STATUS = 'NEW')
c            DO 360 J = 1, NLIFE
c                WRITE(FITFILNUM(I),*) FOS_A(1, I, 1, J), 
c     &              FOS_A(2, I, 1, J), FOS_A(3, I, 1, J), 
c     &              FOS_A(4, I, 1, J), FOS_A(5, I, 1, J),  
c     &              FOS_A(6, I, 1, J), FOS_A(7, I, 1, J),  
c     &              FOS_A(8, I, 1, J), FOS_A(9, I, 1, J)    
c  360       CONTINUE
c  370 CONTINUE


  925 FORMAT(/2X,'B LIVES:    Maximum FoS',/)

  926 FORMAT(2X,F7.5,5X,f13.6)
    
      STOP

      END

C************************************************************************
C   SUBROUTINE INSORT PERFORMS AN INSERTION SORT FOR EACH LIFE CALCULATED
C
C   DATE:       OCTOBER 2012
C
C   VERSION:    2.1
C
C   SUBPROGRAMS:N/A
C
C   INPUTS:     NEWLIF, LIFE, NLIFET
C
C   OUTPUTS:    LIFE
C
C           LIST OF VARIABLES
C
C   I           CONTROLS DO LOOP FOR INSERTION
C   IOUT        OUTPUT DUMP CONTROLLER
C   LIFE()      1-D ARRAY CONTAINING TAIL VALUES OF THE LIVES GENERATED BY THE
C                   PFM TO BE SORTED
C   MAXLIF      MAXIMUM NUMBER OF FATIGUE LIVES ALLOWED FOR BETA, THETA, ALPHA,
C                   CALCULATION
C   NEWLIF      LIFE VALUE TO BE INSERTED INTO LIFE()
C   NLIFET      TOTAL NUMBER OF LIVES CALCULATED BY PFM
C   NUM         NUMBER OF LIFE VALUES IN LIFE()
C   PLACE       POSITION WHERE NEWLIF IS TO BE INSERTED INTO LIFE()
C   TEMP()      1-D ARRAY CONTAINING VALUES OF LIFE() TO BE SHIFTED UPON 
C                   INSERTION OF NEWLIF
C
C************************************************************************
      SUBROUTINE INSORT (NEWFOS, FOSL, NLIFE)

      IMPLICIT NONE

      INTEGER  MAXLIF

      PARAMETER (MAXLIF = 100000)

      COMMON   IOUT

      INTEGER  I, IOUT, NLIFE, NUM, PLACE

      REAL     FOSL(MAXLIF), NEWFOS, TEMP(MAXLIF)

c      NUM = NLIFET / 2.0
       NUM = NLIFE

C   FIND POSITION IN LIFE() FOR NEWLIF
c      IF (NEWLIF .GT. LIFE(NUM)) GOTO 400

      DO 100 I = 1, NUM
         IF (NEWFOS .GT. FOSL(I)) THEN
            PLACE = I
            GOTO 110
         ENDIF
  100 CONTINUE
  110 CONTINUE

C   STORE VALUES OF LIFE() TO BE SHIFTED DUE TO NEWLIF INSERTION IN TEMP()
      DO 200 I = (PLACE + 1), NUM
         TEMP(I) = FOSL(I-1)
  200 CONTINUE

C   INSERT NEWLIF
      FOSL(PLACE) = NEWFOS

C   SHIFT VALUES OF LIFE() FOLLOWING NEWLIF
      DO 300 I = (PLACE + 1), NUM
         FOSL(I) = TEMP(I)
  300 CONTINUE

C   IF NEWLIF IS LARGER THAN ALL LIVES IN LIFE() THEN RETURN
c  400 CONTINUE

      RETURN
      END
            
C************************************************************************
C   SUBROUTINE TRMNAT HANDLES THE TERMINATION OF THE PROGRAM RUN WHEN
C       ONE OF THE PROGRAM'S ASSUMPTIONS HAVE BEEN VIOLATED
C
C************************************************************************
      SUBROUTINE TRMNAT

      WRITE(8,*) 'PROGRAM EXECUTION TERMINATED'
      STOP
      END  

C***********************************************************************
C   SUBROUTINE PARDRW PERFORMS THE RANDOM LIFE DRIVER PARAMETER DRAWS IN 
C       THE INNER LOOP
C
C   DATE :      OCTOBER 2012
C
C************************************************************************
      SUBROUTINE PARDRW (NGEOM, NLDHIS, LAMGEOM, LAMLD, LAMSTR, RAND)

      IMPLICIT NONE

      INTEGER   I, IOUT, MAXNGEOM, MAXNLDHIS, NGEOM, NLDHIS
      
      PARAMETER (MAXNGEOM = 3, MAXNLDHIS = 1)

      DOUBLE PRECISION  RAND

      REAL      DUM, LAMGEOM(MAXNGEOM), 
     &          LAMGEOMA(MAXNGEOM), LAMGEOMB(MAXNGEOM), 
     &          LAMSTR, LAMSTRA, LAMSTRB, 
     &          LAMLD(MAXNLDHIS), LAMLDA(MAXNLDHIS), LAMLDB(MAXNLDHIS)

      COMMON/DRIVRS/    LAMGEOMA,   LAMGEOMB, 
     &                  LAMLDA,     LAMLDB,
     &                  LAMSTRA,    LAMSTRB
     
      COMMON    IOUT

C   PARAMETERS DRAW
      DO 10 I = 1, MAXNGEOM
        CALL PRYRV (RAND, LAMGEOMA(I),LAMGEOMB(I), 
     &               0, 1, LAMGEOM(I), DUM)
   10 CONTINUE
   
C   LOAD DRAW
      DO 20 I = 1, MAXNLDHIS
        CALL PRYRV (RAND, LAMLDA(I), LAMLDB(I), 0, 1, LAMLD(I), DUM)
   20 CONTINUE     
    
C   MATERIAL DRAW   
      CALL PRYRV (RAND, LAMSTRA, LAMSTRB, 0, 0, LAMSTR, DUM)
                
      RETURN
      END
     
C************************************************************************
C   SUBROUTINE INPUT READS IN THE DATA AND ECHOES IT 
C
C   DATE :      OCTOBER 2012
C
C************************************************************************
      SUBROUTINE INPUT (DESCGEOM, BLFPER, FTMEAN, GEOMDATA, NBLIFE, 
     &          NGEOM, NLDHIS, NLIFE, NTIME, PGEOM, PSIG, RAND, SGEOM, 
     &          USTRHIS)
 
      IMPLICIT NONE

      INTEGER   MAXBLF, MAXM, MAXNGEOM, MAXNLDHIS, MAXSGEOM

      PARAMETER (MAXBLF = 35, MAXM = 20000, MAXNGEOM = 3, 
     &           MAXNLDHIS = 1, MAXSGEOM = 11)

      INTEGER   FILNUM(4), I, IOUT, J, MPROC, 
     &          NBLIFE, NGEOM, NLDHIS, NLIFE, NTIME, SGEOM(MAXNGEOM)

      DOUBLE PRECISION  RAND

      REAL      BLFPER(MAXBLF), FTMEAN, GEOMDATA(MAXNGEOM, MAXSGEOM), 
     &          MAXGEOM(MAXNGEOM), MINGEOM(MAXNGEOM),  
     &          LAMGEOMA(MAXNGEOM), LAMGEOMB(MAXNGEOM),
     &          LAMLDA(MAXNLDHIS), LAMLDB(MAXNLDHIS), 
     &          LAMSTRA, LAMSTRB, PGEOM(MAXNGEOM), PSIG,
     &          USTRHIS(MAXNLDHIS, MAXM)
     
      CHARACTER*8 LDNAME(4)
      
      CHARACTER*40 DESCGEOM(MAXNGEOM)
      
      CHARACTER*40 DESCRP   

      COMMON/DRIVRS/    LAMGEOMA,   LAMGEOMB, 
     &                  LAMLDA,     LAMLDB,
     &                  LAMSTRA,    LAMSTRB
     
      COMMON    IOUT
      
      LOGICAL   FTEST
   
      DATA (LDNAME(I), I = 1, 4) /
     &     'STRHIS1','STRHIS2','STRHIS3','STRHIS4'/
     
      DATA (FILNUM(I), I = 1, 4) /
     &     11, 12, 13, 14 /

      READ(1,*) RAND
      WRITE(8,*) '                         RANDOM NUMBER SEED =', RAND
      READ(1,*) NLIFE
      WRITE(8,*) '                                  LOOP SIZE =', NLIFE

      READ(1,*) NBLIFE
      IF (NBLIFE .GT. 0) READ(1,*) (BLFPER(J), J = 1, NBLIFE)

      READ(1,*) LAMSTRA, LAMSTRB
      
      READ(2,*) NLDHIS,  NTIME
      WRITE(8,*) '                    NUMBER OF LOAD HISTORYS =', NLDHIS
      WRITE(8,*) '    NUMBER OF POINTS IN STRESS-TIME HISTORY =', NTIME
      
      IF (NLDHIS .GT. MAXNLDHIS)THEN
        WRITE(8,*) 'ERROR: TOO MANY LOAD HISTORYS'
        CALL TRMNAT
      ENDIF
      
      IF (NTIME .GT. MAXM) THEN
        WRITE(8,*) 'ERROR:  LOAD TIME HISTORY TOO LARGE'
        CALL TRMNAT
      ENDIF
   
      DO 15 I = 1, NLDHIS
        READ(2,*)   LAMLDA(I), LAMLDB(I)
   15 CONTINUE

C ** READ COMPONENT DATA 
      READ(2,*) NGEOM
      WRITE(8,*) '             NUMBER OF PARAMETERS =', NGEOM
      
      IF (NGEOM .GT. MAXNGEOM)THEN
        WRITE(8,*) 'ERROR: TOO MANY PARAMETERS'
        CALL TRMNAT
      ENDIF
      
C   INITIALIZE SGEOM, PGEOM
      DO 11 I = 1, MAXNGEOM
        SGEOM(I) = 1
        PGEOM(I) = 0
   11 CONTINUE
            
      DO 10 I = 1, NGEOM
        READ(2,*)   DESCGEOM(I), PGEOM(I), MINGEOM(I), MAXGEOM(I), 
     &              LAMGEOMA(I), LAMGEOMB(I), SGEOM(I)
        IF ((SGEOM(I) .EQ. 1) .AND. 
     &           (MAXGEOM(I) .NE. MINGEOM(I))) THEN
            WRITE(8,*) 'ERROR: SGEOM =1 WHEN MAXGEOM = MINGEOM'
            CALL TRMNAT
        ENDIF
        IF (SGEOM(I) .GT. MAXSGEOM) THEN
            WRITE(8,*) 'ERROR: TOO MANY SEGMENTS FOR PARAMETER ', NGEOM
            CALL TRMNAT
        ELSE IF (SGEOM(I) .LE. 0) THEN
            WRITE(8,*) 'ERROR: NEGATIVE SEGMENT FOR PARAMETER ', NGEOM
            CALL TRMNAT
        ENDIF
   10 CONTINUE
   
C CALCULATE INDIVIDUAL POINT FOR EACH PARAMETERS    
      DO 60 I = 1, NGEOM
        DO 70 J = 1, SGEOM(I)
            IF ( MAXGEOM(I) .EQ. MINGEOM(I) ) THEN
                GEOMDATA(I, J) = MAXGEOM(I)
            ELSE
                GEOMDATA(I, J) = (J-1) * ( MAXGEOM(I) - MINGEOM(I) )
     &                  / (SGEOM(I)-1) + MINGEOM(I)
            ENDIF
   70   CONTINUE                    
   60 CONTINUE
   
C   INPUT DATA ON SPECIFIC MATERIAL FROM SPECFD AND ECHO TO SPECFO
      READ(2,*) DESCRP
      READ(2,*) FTMEAN, PSIG 
                  
C  READ LOAD TIME HISTORY FROM FILES
      DO 40 I = 1, NLDHIS
         INQUIRE (FILE = LDNAME(I), EXIST = FTEST)
         IF (FTEST .EQV. .TRUE.) THEN
            OPEN (FILNUM(I), FILE = LDNAME(I), STATUS = 'OLD')
            DO 35 J = 1, NTIME
               READ(FILNUM(I),*) USTRHIS(I, J)
   35       CONTINUE
            CLOSE (FILNUM(I))
         ELSE
            WRITE(8,*) 'ERROR: CANNOT OPEN FILE, ', LDNAME(I), 
     &                ' DOES NOT EXIST'
            CALL TRMNAT
         ENDIF
   40 CONTINUE
            
C   ECHO DATA TO ELFOUT
      WRITE(3,900)
      
      DO 45 I = 1, NGEOM
        WRITE(3, 901) I, DESCGEOM(I), MINGEOM(I),MAXGEOM(I), 
     &                   LAMGEOMA(I), LAMGEOMB(I), SGEOM(I)
   45 CONTINUE
   
      DO 160 I = 1, NGEOM
        WRITE(3, *), ' PARAMETER #',I, DESCGEOM(I)
        DO 170 J = 1, SGEOM(I)
            WRITE(3, *), GEOMDATA(I, J)  
  170   CONTINUE                    
  160 CONTINUE
        
      DO 50 I = 1, NLDHIS
        WRITE(3,903) I, LAMLDA(I), LAMLDB(I), NTIME
   50 CONTINUE
            
      WRITE(3,904) LAMSTRA, LAMSTRB 
          
C   FORMAT STATEMENTS TO ECHO INPUT DATA TO SROUT
  900 FORMAT(28X,'ANGELES CREST ENGINEERING INC',//,
     &       28X,'STRESS RUPTURE',//
     &       36X,'INPUT DATA',
     &       ///,14X,'DRIVERS',28X,'PARAMETER DISTRIBUTIONS',//)

  901 FORMAT( 2X, 'PARAMETER #', I1, 5X, A40/,
     &        2X, 'LOWER BOUND = ', F9.5, 10X, 'UPPER BOUND = ', F9.5,/,
     &        2X, 'UNCERTAINTY  ',21X,   'U(',F9.5,',',F9.5,')',/,
     &        2X, 'NUMBER OF PARAMETER POINTS = 'I2,/)

  903 FORMAT(//,2X, 'LOAD HISTORY #', I1,/,
     &          2X  'UNCERTAINTY', 21X, 'U(',F9.5,',',F9.5,')',/,
     &          2X, 'NUMBER OF DATA POINTS = ' I4,/)
     
  904 FORMAT(//,2X,'STRESS ANALYSIS UNCERTAINTY',21X,
     &       'U(',F8.5,',',F9.5,')')
     
      RETURN
      END
     
C************************************************************************ 
C   SUBROUTINE STRAN PERFORMS THE UNIT STRESS TO STRESS TRANSFORMATION 
C               AND OUTPUT THE MAX STRESS
C
C   DATE:       OCTOBER 2012
C************************************************************************ 
      SUBROUTINE STRAN (GEOM, NGEOM, NLDHIS, LAMGEOM, LAMLD,  
     &                  LAMSTR, NTIME, PGEOM, MAXSTRESS, USTRHIS)
 
      INTEGER  MAXM, MAXNGEOM, MAXNLDHIS
 
      PARAMETER (MAXM = 20000, MAXNGEOM = 3, MAXNLDHIS = 1) 
 
      COMMON  IOUT 
 
      INTEGER  I, IOUT, J, K, NGEOM, NLDHIS, NTIME 
 
      REAL     GEOM(MAXNGEOM), LAMGEOM(MAXNGEOM), LAMLD(MAXNLDHIS), 
     &         LAMSTR, PGEOM(MAXNGEOM), STRHIS(MAXM), 
     &         USTRHIS(MAXNLDHIS,MAXM), MAXSTRESS
 
C   DERIVE STRESS HISTORY FROM UNIT STRESS HISTORY
      DO 100 I = 1, NTIME
        STRHIS(I) = USTRHIS(1, I) * ( LAMLD(1) * LAMSTR ) 
        DO 200 J = 1, NGEOM
            STRHIS(I) = STRHIS(I) * 
     &                  ( ( GEOM(J) * LAMGEOM(J) ) ** PGEOM(J) )
  200   CONTINUE
        IF (STRHIS(I) .GT. MAXSTRESS)   THEN
            MAXSTRESS = STRHIS(I)
        ENDIF              
  100 CONTINUE   
       
      RETURN  
      
      END 
